@@ -42,6 +42,8 @@ typedef struct _ValaCCodeBinaryExpression ValaCCodeBinaryExpression;
 typedef struct _ValaCCodeBinaryExpressionClass ValaCCodeBinaryExpressionClass;
 typedef struct _ValaCCodeBinaryExpressionPrivate ValaCCodeBinaryExpressionPrivate;
 
+#define VALA_TYPE_CCODE_BINARY_OPERATOR (vala_ccode_binary_operator_get_type ())
+
 /**
  * Represents an expression with two operands in C code.
  */
@@ -73,6 +75,7 @@ typedef enum  {
 	VALA_CCODE_BINARY_OPERATOR_OR
 } ValaCCodeBinaryOperator;
 
+
 ValaCCodeBinaryExpression* vala_ccode_binary_expression_new (ValaCCodeBinaryOperator op, ValaCCodeExpression* l, ValaCCodeExpression* r);
 ValaCCodeBinaryOperator vala_ccode_binary_expression_get_operator (ValaCCodeBinaryExpression* self);
 void vala_ccode_binary_expression_set_operator (ValaCCodeBinaryExpression* self, ValaCCodeBinaryOperator value);
@@ -81,6 +84,8 @@ void vala_ccode_binary_expression_set_left (ValaCCodeBinaryExpression* self, Val
 ValaCCodeExpression* vala_ccode_binary_expression_get_right (ValaCCodeBinaryExpression* self);
 void vala_ccode_binary_expression_set_right (ValaCCodeBinaryExpression* self, ValaCCodeExpression* value);
 GType vala_ccode_binary_expression_get_type (void);
+GType vala_ccode_binary_operator_get_type (void);
+
 
 G_END_DECLS
 
