@@ -24,6 +24,14 @@
 #include <gee/arraylist.h>
 #include <gee/list.h>
 #include <gee/collection.h>
+#include <ccode/valaccodelabel.h>
+#include <ccode/valaccodereturnstatement.h>
+#include <ccode/valaccodegotostatement.h>
+#include <ccode/valaccodecontinuestatement.h>
+#include <ccode/valaccodebreakstatement.h>
+
+
+
 
 struct _ValaCCodeBlockPrivate {
 	gboolean _suppress_newline;
@@ -39,12 +47,13 @@ static gpointer vala_ccode_block_parent_class = NULL;
 static void vala_ccode_block_dispose (GObject * obj);
 
 
+
 /**
  * Prepend the specified statement to the list of statements.
  */
 void vala_ccode_block_prepend_statement (ValaCCodeBlock* self, ValaCCodeNode* statement) {
 	g_return_if_fail (VALA_IS_CCODE_BLOCK (self));
-	g_return_if_fail (VALA_IS_CCODE_NODE (statement));
+	g_return_if_fail (statement == NULL || VALA_IS_CCODE_NODE (statement));
 	gee_list_insert (self->priv->statements, 0, statement);
 }
 
@@ -54,7 +63,7 @@ void vala_ccode_block_prepend_statement (ValaCCodeBlock* self, ValaCCodeNode* st
  */
 void vala_ccode_block_add_statement (ValaCCodeBlock* self, ValaCCodeNode* statement) {
 	g_return_if_fail (VALA_IS_CCODE_BLOCK (self));
-	g_return_if_fail (VALA_IS_CCODE_NODE (statement));
+	g_return_if_fail (statement == NULL || VALA_IS_CCODE_NODE (statement));
 	/* allow generic nodes to include comments */
 	gee_collection_add (GEE_COLLECTION (self->priv->statements), statement);
 }
@@ -62,8 +71,11 @@ void vala_ccode_block_add_statement (ValaCCodeBlock* self, ValaCCodeNode* statem
 
 static void vala_ccode_block_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeBlock * self;
+	ValaCCodeNode* last_statement;
 	self = VALA_CCODE_BLOCK (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
+	/* the last reachable statement*/
+	last_statement = NULL;
 	vala_ccode_writer_write_begin_block (writer);
 	{
 		GeeList* statement_collection;
@@ -72,9 +84,23 @@ static void vala_ccode_block_real_write (ValaCCodeNode* base, ValaCCodeWriter* w
 		statement_it = gee_iterable_iterator (GEE_ITERABLE (statement_collection));
 		while (gee_iterator_next (statement_it)) {
 			ValaCCodeNode* statement;
-			statement = gee_iterator_get (statement_it);
+			statement = ((ValaCCodeNode*) gee_iterator_get (statement_it));
 			{
 				vala_ccode_node_write_declaration (statement, writer);
+				/* determine last reachable statement*/
+				if (VALA_IS_CCODE_LABEL (statement)) {
+					ValaCCodeNode* _tmp0;
+					_tmp0 = NULL;
+					last_statement = (_tmp0 = NULL, (last_statement == NULL ? NULL : (last_statement = (g_object_unref (last_statement), NULL))), _tmp0);
+				} else {
+					if (VALA_IS_CCODE_RETURN_STATEMENT (statement) || VALA_IS_CCODE_GOTO_STATEMENT (statement) || VALA_IS_CCODE_CONTINUE_STATEMENT (statement) || VALA_IS_CCODE_BREAK_STATEMENT (statement)) {
+						ValaCCodeNode* _tmp2;
+						ValaCCodeNode* _tmp1;
+						_tmp2 = NULL;
+						_tmp1 = NULL;
+						last_statement = (_tmp2 = (_tmp1 = statement, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (last_statement == NULL ? NULL : (last_statement = (g_object_unref (last_statement), NULL))), _tmp2);
+					}
+				}
 				(statement == NULL ? NULL : (statement = (g_object_unref (statement), NULL)));
 			}
 		}
@@ -87,18 +113,24 @@ static void vala_ccode_block_real_write (ValaCCodeNode* base, ValaCCodeWriter* w
 		statement_it = gee_iterable_iterator (GEE_ITERABLE (statement_collection));
 		while (gee_iterator_next (statement_it)) {
 			ValaCCodeNode* statement;
-			statement = gee_iterator_get (statement_it);
+			statement = ((ValaCCodeNode*) gee_iterator_get (statement_it));
 			{
 				vala_ccode_node_write (statement, writer);
+				/* only output reachable code*/
+				if (statement == last_statement) {
+					(statement == NULL ? NULL : (statement = (g_object_unref (statement), NULL)));
+					break;
+				}
 				(statement == NULL ? NULL : (statement = (g_object_unref (statement), NULL)));
 			}
 		}
 		(statement_it == NULL ? NULL : (statement_it = (g_object_unref (statement_it), NULL)));
 	}
 	vala_ccode_writer_write_end_block (writer);
-	if (!vala_ccode_block_get_suppress_newline (self)) {
+	if (!self->priv->_suppress_newline) {
 		vala_ccode_writer_write_newline (writer);
 	}
+	(last_statement == NULL ? NULL : (last_statement = (g_object_unref (last_statement), NULL)));
 }
 
 
