@@ -29,6 +29,9 @@
 #include <ccode/valaccodenode.h>
 #include <ccode/valaccodelinedirective.h>
 
+
+
+
 struct _ValaCCodeCaseStatementPrivate {
 	ValaCCodeExpression* _expression;
 	GeeList* statements;
@@ -43,9 +46,10 @@ static gpointer vala_ccode_case_statement_parent_class = NULL;
 static void vala_ccode_case_statement_dispose (GObject * obj);
 
 
+
 ValaCCodeCaseStatement* vala_ccode_case_statement_new (ValaCCodeExpression* expression) {
 	ValaCCodeCaseStatement * self;
-	g_return_val_if_fail (VALA_IS_CCODE_EXPRESSION (expression), NULL);
+	g_return_val_if_fail (expression == NULL || VALA_IS_CCODE_EXPRESSION (expression), NULL);
 	self = g_object_newv (VALA_TYPE_CCODE_CASE_STATEMENT, 0, NULL);
 	vala_ccode_case_statement_set_expression (self, expression);
 	return self;
@@ -59,7 +63,7 @@ ValaCCodeCaseStatement* vala_ccode_case_statement_new (ValaCCodeExpression* expr
  */
 void vala_ccode_case_statement_add_statement (ValaCCodeCaseStatement* self, ValaCCodeStatement* stmt) {
 	g_return_if_fail (VALA_IS_CCODE_CASE_STATEMENT (self));
-	g_return_if_fail (VALA_IS_CCODE_STATEMENT (stmt));
+	g_return_if_fail (stmt == NULL || VALA_IS_CCODE_STATEMENT (stmt));
 	gee_collection_add (GEE_COLLECTION (self->priv->statements), stmt);
 }
 
@@ -67,10 +71,10 @@ void vala_ccode_case_statement_add_statement (ValaCCodeCaseStatement* self, Vala
 static void vala_ccode_case_statement_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeCaseStatement * self;
 	self = VALA_CCODE_CASE_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	vala_ccode_writer_write_indent (writer, vala_ccode_node_get_line (VALA_CCODE_NODE (self)));
 	vala_ccode_writer_write_string (writer, "case ");
-	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_case_statement_get_expression (self)), writer);
+	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->_expression), writer);
 	vala_ccode_writer_write_string (writer, ":");
 	vala_ccode_writer_write_newline (writer);
 	{
@@ -80,7 +84,7 @@ static void vala_ccode_case_statement_real_write (ValaCCodeNode* base, ValaCCode
 		stmt_it = gee_iterable_iterator (GEE_ITERABLE (stmt_collection));
 		while (gee_iterator_next (stmt_it)) {
 			ValaCCodeStatement* stmt;
-			stmt = gee_iterator_get (stmt_it);
+			stmt = ((ValaCCodeStatement*) gee_iterator_get (stmt_it));
 			{
 				vala_ccode_node_write (VALA_CCODE_NODE (stmt), writer);
 				(stmt == NULL ? NULL : (stmt = (g_object_unref (stmt), NULL)));
@@ -98,10 +102,12 @@ ValaCCodeExpression* vala_ccode_case_statement_get_expression (ValaCCodeCaseStat
 
 
 void vala_ccode_case_statement_set_expression (ValaCCodeCaseStatement* self, ValaCCodeExpression* value) {
+	ValaCCodeExpression* _tmp2;
 	ValaCCodeExpression* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_CASE_STATEMENT (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_expression = (_tmp1 = g_object_ref (value), (self->priv->_expression == NULL ? NULL : (self->priv->_expression = (g_object_unref (self->priv->_expression), NULL))), _tmp1);
+	self->priv->_expression = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_expression == NULL ? NULL : (self->priv->_expression = (g_object_unref (self->priv->_expression), NULL))), _tmp2);
 }
 
 
