@@ -22,6 +22,9 @@
 
 #include <ccode/valaccodecomment.h>
 
+
+
+
 struct _ValaCCodeCommentPrivate {
 	char* _text;
 };
@@ -35,6 +38,7 @@ static gpointer vala_ccode_comment_parent_class = NULL;
 static void vala_ccode_comment_dispose (GObject * obj);
 
 
+
 ValaCCodeComment* vala_ccode_comment_new (const char* _text) {
 	GParameter * __params;
 	GParameter * __params_it;
@@ -59,7 +63,7 @@ static void vala_ccode_comment_real_write (ValaCCodeNode* base, ValaCCodeWriter*
 	ValaCCodeComment * self;
 	self = VALA_CCODE_COMMENT (base);
 	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
-	vala_ccode_writer_write_comment (writer, vala_ccode_comment_get_text (self));
+	vala_ccode_writer_write_comment (writer, self->priv->_text);
 }
 
 
