@@ -47,11 +47,11 @@ typedef struct _ValaCCodeDoStatementPrivate ValaCCodeDoStatementPrivate;
  * Represents a do iteration statement in the C code.
  */
 struct _ValaCCodeDoStatement {
-	ValaCCodeStatement parent;
+	ValaCCodeStatement parent_instance;
 	ValaCCodeDoStatementPrivate * priv;
 };
 struct _ValaCCodeDoStatementClass {
-	ValaCCodeStatementClass parent;
+	ValaCCodeStatementClass parent_class;
 };
 
 ValaCCodeDoStatement* vala_ccode_do_statement_new (ValaCCodeStatement* stmt, ValaCCodeExpression* cond);
@@ -61,6 +61,7 @@ ValaCCodeExpression* vala_ccode_do_statement_get_condition (ValaCCodeDoStatement
 void vala_ccode_do_statement_set_condition (ValaCCodeDoStatement* self, ValaCCodeExpression* value);
 GType vala_ccode_do_statement_get_type (void);
 
+
 G_END_DECLS
 
 #endif
