@@ -22,6 +22,9 @@
 
 #include <ccode/valaccodeenumvalue.h>
 
+
+
+
 struct _ValaCCodeEnumValuePrivate {
 	char* _name;
 	ValaCCodeExpression* _value;
@@ -37,6 +40,7 @@ static gpointer vala_ccode_enum_value_parent_class = NULL;
 static void vala_ccode_enum_value_dispose (GObject * obj);
 
 
+
 ValaCCodeEnumValue* vala_ccode_enum_value_new (const char* name, ValaCCodeExpression* value) {
 	ValaCCodeEnumValue * self;
 	g_return_val_if_fail (value == NULL || VALA_IS_CCODE_EXPRESSION (value), NULL);
@@ -51,10 +55,10 @@ static void vala_ccode_enum_value_real_write (ValaCCodeNode* base, ValaCCodeWrit
 	ValaCCodeEnumValue * self;
 	self = VALA_CCODE_ENUM_VALUE (base);
 	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
-	vala_ccode_writer_write_string (writer, vala_ccode_enum_value_get_name (self));
-	if (vala_ccode_enum_value_get_value (self) != NULL) {
+	vala_ccode_writer_write_string (writer, self->priv->_name);
+	if (self->priv->_value != NULL) {
 		vala_ccode_writer_write_string (writer, " = ");
-		vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_enum_value_get_value (self)), writer);
+		vala_ccode_node_write (VALA_CCODE_NODE (self->priv->_value), writer);
 	}
 }
 
