@@ -47,11 +47,11 @@ typedef struct _ValaCCodeForStatementPrivate ValaCCodeForStatementPrivate;
  * Represents a for iteration statement in the C code.
  */
 struct _ValaCCodeForStatement {
-	ValaCCodeStatement parent;
+	ValaCCodeStatement parent_instance;
 	ValaCCodeForStatementPrivate * priv;
 };
 struct _ValaCCodeForStatementClass {
-	ValaCCodeStatementClass parent;
+	ValaCCodeStatementClass parent_class;
 };
 
 ValaCCodeForStatement* vala_ccode_for_statement_new (ValaCCodeExpression* condition, ValaCCodeStatement* body);
@@ -63,6 +63,7 @@ ValaCCodeStatement* vala_ccode_for_statement_get_body (ValaCCodeForStatement* se
 void vala_ccode_for_statement_set_body (ValaCCodeForStatement* self, ValaCCodeStatement* value);
 GType vala_ccode_for_statement_get_type (void);
 
+
 G_END_DECLS
 
 #endif
