@@ -26,6 +26,9 @@
 #include <gee/collection.h>
 #include <ccode/valaccodelinedirective.h>
 
+
+
+
 struct _ValaCCodeFunctionPrivate {
 	char* _name;
 	ValaCCodeModifiers _modifiers;
@@ -46,6 +49,7 @@ static gpointer vala_ccode_function_parent_class = NULL;
 static void vala_ccode_function_dispose (GObject * obj);
 
 
+
 ValaCCodeFunction* vala_ccode_function_new (const char* name, const char* return_type) {
 	ValaCCodeFunction * self;
 	self = g_object_newv (VALA_TYPE_CCODE_FUNCTION, 0, NULL);
@@ -75,8 +79,8 @@ void vala_ccode_function_add_parameter (ValaCCodeFunction* self, ValaCCodeFormal
 ValaCCodeFunction* vala_ccode_function_copy (ValaCCodeFunction* self) {
 	ValaCCodeFunction* func;
 	g_return_val_if_fail (VALA_IS_CCODE_FUNCTION (self), NULL);
-	func = vala_ccode_function_new (vala_ccode_function_get_name (self), vala_ccode_function_get_return_type (self));
-	vala_ccode_function_set_modifiers (func, vala_ccode_function_get_modifiers (self));
+	func = vala_ccode_function_new (self->priv->_name, self->priv->_return_type);
+	vala_ccode_function_set_modifiers (func, self->priv->_modifiers);
 	{
 		GeeList* param_collection;
 		GeeIterator* param_it;
@@ -84,7 +88,7 @@ ValaCCodeFunction* vala_ccode_function_copy (ValaCCodeFunction* self) {
 		param_it = gee_iterable_iterator (GEE_ITERABLE (param_collection));
 		while (gee_iterator_next (param_it)) {
 			ValaCCodeFormalParameter* param;
-			param = gee_iterator_get (param_it);
+			param = ((ValaCCodeFormalParameter*) gee_iterator_get (param_it));
 			{
 				/* no deep copy for lists available yet
 				 * func.parameters = parameters.copy ();
@@ -95,7 +99,7 @@ ValaCCodeFunction* vala_ccode_function_copy (ValaCCodeFunction* self) {
 		}
 		(param_it == NULL ? NULL : (param_it = (g_object_unref (param_it), NULL)));
 	}
-	vala_ccode_function_set_block (func, vala_ccode_function_get_block (self));
+	vala_ccode_function_set_block (func, self->priv->_block);
 	return func;
 }
 
@@ -106,15 +110,15 @@ static void vala_ccode_function_real_write (ValaCCodeNode* base, ValaCCodeWriter
 	self = VALA_CCODE_FUNCTION (base);
 	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	vala_ccode_writer_write_indent (writer, vala_ccode_node_get_line (VALA_CCODE_NODE (self)));
-	if (((vala_ccode_function_get_modifiers (self)) & (VALA_CCODE_MODIFIERS_STATIC)) == (VALA_CCODE_MODIFIERS_STATIC)) {
+	if (((self->priv->_modifiers) & (VALA_CCODE_MODIFIERS_STATIC)) == (VALA_CCODE_MODIFIERS_STATIC)) {
 		vala_ccode_writer_write_string (writer, "static ");
 	}
-	if (((vala_ccode_function_get_modifiers (self)) & (VALA_CCODE_MODIFIERS_INLINE)) == (VALA_CCODE_MODIFIERS_INLINE)) {
+	if (((self->priv->_modifiers) & (VALA_CCODE_MODIFIERS_INLINE)) == (VALA_CCODE_MODIFIERS_INLINE)) {
 		vala_ccode_writer_write_string (writer, "inline ");
 	}
-	vala_ccode_writer_write_string (writer, vala_ccode_function_get_return_type (self));
+	vala_ccode_writer_write_string (writer, self->priv->_return_type);
 	vala_ccode_writer_write_string (writer, " ");
-	vala_ccode_writer_write_string (writer, vala_ccode_function_get_name (self));
+	vala_ccode_writer_write_string (writer, self->priv->_name);
 	vala_ccode_writer_write_string (writer, " (");
 	first = TRUE;
 	{
@@ -124,7 +128,7 @@ static void vala_ccode_function_real_write (ValaCCodeNode* base, ValaCCodeWriter
 		param_it = gee_iterable_iterator (GEE_ITERABLE (param_collection));
 		while (gee_iterator_next (param_it)) {
 			ValaCCodeFormalParameter* param;
-			param = gee_iterator_get (param_it);
+			param = ((ValaCCodeFormalParameter*) gee_iterator_get (param_it));
 			{
 				if (!first) {
 					vala_ccode_writer_write_string (writer, ", ");
@@ -141,10 +145,10 @@ static void vala_ccode_function_real_write (ValaCCodeNode* base, ValaCCodeWriter
 		vala_ccode_writer_write_string (writer, "void");
 	}
 	vala_ccode_writer_write_string (writer, ")");
-	if (vala_ccode_function_get_block (self) == NULL) {
+	if (self->priv->_block == NULL) {
 		vala_ccode_writer_write_string (writer, ";");
 	} else {
-		vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_function_get_block (self)), writer);
+		vala_ccode_node_write (VALA_CCODE_NODE (self->priv->_block), writer);
 		vala_ccode_writer_write_newline (writer);
 	}
 	vala_ccode_writer_write_newline (writer);
@@ -219,7 +223,7 @@ static void vala_ccode_function_get_property (GObject * object, guint property_i
 		g_value_set_string (value, vala_ccode_function_get_name (self));
 		break;
 		case VALA_CCODE_FUNCTION_MODIFIERS:
-		g_value_set_int (value, vala_ccode_function_get_modifiers (self));
+		g_value_set_flags (value, vala_ccode_function_get_modifiers (self));
 		break;
 		case VALA_CCODE_FUNCTION_RETURN_TYPE:
 		g_value_set_string (value, vala_ccode_function_get_return_type (self));
@@ -242,7 +246,7 @@ static void vala_ccode_function_set_property (GObject * object, guint property_i
 		vala_ccode_function_set_name (self, g_value_get_string (value));
 		break;
 		case VALA_CCODE_FUNCTION_MODIFIERS:
-		vala_ccode_function_set_modifiers (self, g_value_get_int (value));
+		vala_ccode_function_set_modifiers (self, g_value_get_flags (value));
 		break;
 		case VALA_CCODE_FUNCTION_RETURN_TYPE:
 		vala_ccode_function_set_return_type (self, g_value_get_string (value));
@@ -265,7 +269,7 @@ static void vala_ccode_function_class_init (ValaCCodeFunctionClass * klass) {
 	G_OBJECT_CLASS (klass)->dispose = vala_ccode_function_dispose;
 	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_function_real_write;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_MODIFIERS, g_param_spec_int ("modifiers", "modifiers", "modifiers", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_MODIFIERS, g_param_spec_flags ("modifiers", "modifiers", "modifiers", VALA_TYPE_CCODE_MODIFIERS, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_RETURN_TYPE, g_param_spec_string ("return-type", "return-type", "return-type", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CCODE_FUNCTION_BLOCK, g_param_spec_object ("block", "block", "block", VALA_TYPE_CCODE_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
@@ -280,8 +284,8 @@ static void vala_ccode_function_init (ValaCCodeFunction * self) {
 static void vala_ccode_function_dispose (GObject * obj) {
 	ValaCCodeFunction * self;
 	self = VALA_CCODE_FUNCTION (obj);
-	(self->priv->_name = (g_free (self->priv->_name), NULL));
-	(self->priv->_return_type = (g_free (self->priv->_return_type), NULL));
+	self->priv->_name = (g_free (self->priv->_name), NULL);
+	self->priv->_return_type = (g_free (self->priv->_return_type), NULL);
 	(self->priv->_block == NULL ? NULL : (self->priv->_block = (g_object_unref (self->priv->_block), NULL)));
 	(self->priv->parameters == NULL ? NULL : (self->priv->parameters = (g_object_unref (self->priv->parameters), NULL)));
 	G_OBJECT_CLASS (vala_ccode_function_parent_class)->dispose (obj);
