@@ -28,6 +28,9 @@
 #include <string.h>
 #include <ccode/valaccodenode.h>
 
+
+
+
 struct _ValaCCodeFunctionCallPrivate {
 	ValaCCodeExpression* _call;
 	GeeList* arguments;
@@ -42,6 +45,7 @@ static gpointer vala_ccode_function_call_parent_class = NULL;
 static void vala_ccode_function_call_dispose (GObject * obj);
 
 
+
 ValaCCodeFunctionCall* vala_ccode_function_call_new (ValaCCodeExpression* call) {
 	ValaCCodeFunctionCall * self;
 	g_return_val_if_fail (call == NULL || VALA_IS_CCODE_EXPRESSION (call), NULL);
@@ -58,7 +62,7 @@ ValaCCodeFunctionCall* vala_ccode_function_call_new (ValaCCodeExpression* call)
  */
 void vala_ccode_function_call_add_argument (ValaCCodeFunctionCall* self, ValaCCodeExpression* expr) {
 	g_return_if_fail (VALA_IS_CCODE_FUNCTION_CALL (self));
-	g_return_if_fail (VALA_IS_CCODE_EXPRESSION (expr));
+	g_return_if_fail (expr == NULL || VALA_IS_CCODE_EXPRESSION (expr));
 	gee_collection_add (GEE_COLLECTION (self->priv->arguments), expr);
 }
 
@@ -78,8 +82,8 @@ static void vala_ccode_function_call_real_write (ValaCCodeNode* base, ValaCCodeW
 	ValaCCodeFunctionCall * self;
 	gboolean first;
 	self = VALA_CCODE_FUNCTION_CALL (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
-	vala_ccode_node_write (VALA_CCODE_NODE (vala_ccode_function_call_get_call (self)), writer);
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
+	vala_ccode_node_write (VALA_CCODE_NODE (self->priv->_call), writer);
 	vala_ccode_writer_write_string (writer, " (");
 	first = TRUE;
 	{
@@ -89,7 +93,7 @@ static void vala_ccode_function_call_real_write (ValaCCodeNode* base, ValaCCodeW
 		expr_it = gee_iterable_iterator (GEE_ITERABLE (expr_collection));
 		while (gee_iterator_next (expr_it)) {
 			ValaCCodeExpression* expr;
-			expr = gee_iterator_get (expr_it);
+			expr = ((ValaCCodeExpression*) gee_iterator_get (expr_it));
 			{
 				if (!first) {
 					vala_ccode_writer_write_string (writer, ", ");
