@@ -22,6 +22,9 @@
 
 #include <ccode/valaccodenewline.h>
 
+
+
+
 enum  {
 	VALA_CCODE_NEWLINE_DUMMY_PROPERTY
 };
@@ -29,10 +32,11 @@ static void vala_ccode_newline_real_write (ValaCCodeNode* base, ValaCCodeWriter*
 static gpointer vala_ccode_newline_parent_class = NULL;
 
 
+
 static void vala_ccode_newline_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeNewline * self;
 	self = VALA_CCODE_NEWLINE (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	vala_ccode_writer_write_newline (writer);
 }
 
