@@ -27,6 +27,9 @@
 #include <ccode/valaccodevariabledeclarator.h>
 #include <ccode/valaccodedeclarator.h>
 
+
+
+
 struct _ValaCCodeStructPrivate {
 	char* _name;
 	GeeList* declarations;
@@ -41,9 +44,9 @@ static gpointer vala_ccode_struct_parent_class = NULL;
 static void vala_ccode_struct_dispose (GObject * obj);
 
 
+
 ValaCCodeStruct* vala_ccode_struct_new (const char* name) {
 	ValaCCodeStruct * self;
-	g_return_val_if_fail (name != NULL, NULL);
 	self = g_object_newv (VALA_TYPE_CCODE_STRUCT, 0, NULL);
 	vala_ccode_struct_set_name (self, name);
 	return self;
@@ -57,7 +60,7 @@ ValaCCodeStruct* vala_ccode_struct_new (const char* name) {
  */
 void vala_ccode_struct_add_declaration (ValaCCodeStruct* self, ValaCCodeDeclaration* decl) {
 	g_return_if_fail (VALA_IS_CCODE_STRUCT (self));
-	g_return_if_fail (VALA_IS_CCODE_DECLARATION (decl));
+	g_return_if_fail (decl == NULL || VALA_IS_CCODE_DECLARATION (decl));
 	gee_collection_add (GEE_COLLECTION (self->priv->declarations), decl);
 }
 
@@ -72,8 +75,6 @@ void vala_ccode_struct_add_field (ValaCCodeStruct* self, const char* type_name,
 	ValaCCodeDeclaration* decl;
 	ValaCCodeVariableDeclarator* _tmp0;
 	g_return_if_fail (VALA_IS_CCODE_STRUCT (self));
-	g_return_if_fail (type_name != NULL);
-	g_return_if_fail (name != NULL);
 	decl = vala_ccode_declaration_new (type_name);
 	_tmp0 = NULL;
 	vala_ccode_declaration_add_declarator (decl, VALA_CCODE_DECLARATOR ((_tmp0 = vala_ccode_variable_declarator_new (name))));
@@ -86,9 +87,9 @@ void vala_ccode_struct_add_field (ValaCCodeStruct* self, const char* type_name,
 static void vala_ccode_struct_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
 	ValaCCodeStruct * self;
 	self = VALA_CCODE_STRUCT (base);
-	g_return_if_fail (VALA_IS_CCODE_WRITER (writer));
+	g_return_if_fail (writer == NULL || VALA_IS_CCODE_WRITER (writer));
 	vala_ccode_writer_write_string (writer, "struct ");
-	vala_ccode_writer_write_string (writer, vala_ccode_struct_get_name (self));
+	vala_ccode_writer_write_string (writer, self->priv->_name);
 	vala_ccode_writer_write_begin_block (writer);
 	{
 		GeeList* decl_collection;
@@ -97,7 +98,7 @@ static void vala_ccode_struct_real_write (ValaCCodeNode* base, ValaCCodeWriter*
 		decl_it = gee_iterable_iterator (GEE_ITERABLE (decl_collection));
 		while (gee_iterator_next (decl_it)) {
 			ValaCCodeDeclaration* decl;
-			decl = gee_iterator_get (decl_it);
+			decl = ((ValaCCodeDeclaration*) gee_iterator_get (decl_it));
 			{
 				vala_ccode_node_write_declaration (VALA_CCODE_NODE (decl), writer);
 				(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
@@ -118,10 +119,12 @@ char* vala_ccode_struct_get_name (ValaCCodeStruct* self) {
 
 
 void vala_ccode_struct_set_name (ValaCCodeStruct* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_CCODE_STRUCT (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_name = (_tmp1 = g_strdup (value), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp1);
+	self->priv->_name = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_name = (g_free (self->priv->_name), NULL)), _tmp2);
 }
 
 
@@ -173,7 +176,7 @@ static void vala_ccode_struct_init (ValaCCodeStruct * self) {
 static void vala_ccode_struct_dispose (GObject * obj) {
 	ValaCCodeStruct * self;
 	self = VALA_CCODE_STRUCT (obj);
-	(self->priv->_name = (g_free (self->priv->_name), NULL));
+	self->priv->_name = (g_free (self->priv->_name), NULL);
 	(self->priv->declarations == NULL ? NULL : (self->priv->declarations = (g_object_unref (self->priv->declarations), NULL)));
 	G_OBJECT_CLASS (vala_ccode_struct_parent_class)->dispose (obj);
 }
