@@ -45,11 +45,11 @@ typedef struct _ValaCCodeWriterPrivate ValaCCodeWriterPrivate;
  * Represents a writer to write C source files.
  */
 struct _ValaCCodeWriter {
-	GObject parent;
+	GObject parent_instance;
 	ValaCCodeWriterPrivate * priv;
 };
 struct _ValaCCodeWriterClass {
-	GObjectClass parent;
+	GObjectClass parent_class;
 };
 
 ValaCCodeWriter* vala_ccode_writer_new (const char* _filename);
@@ -61,12 +61,12 @@ void vala_ccode_writer_write_begin_block (ValaCCodeWriter* self);
 void vala_ccode_writer_write_end_block (ValaCCodeWriter* self);
 void vala_ccode_writer_write_comment (ValaCCodeWriter* self, const char* text);
 char* vala_ccode_writer_get_filename (ValaCCodeWriter* self);
-void vala_ccode_writer_set_filename (ValaCCodeWriter* self, const char* value);
 gboolean vala_ccode_writer_get_line_directives (ValaCCodeWriter* self);
 void vala_ccode_writer_set_line_directives (ValaCCodeWriter* self, gboolean value);
 gboolean vala_ccode_writer_get_bol (ValaCCodeWriter* self);
 GType vala_ccode_writer_get_type (void);
 
+
 G_END_DECLS
 
 #endif
