@@ -1,6 +1,6 @@
 /* valadbusbindingprovider.vala
  *
- * Copyright (C) 2007  Jürg Billeter
+ * Copyright (C) 2007-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,11 +21,12 @@
  */
 
 #include <gobject/valadbusbindingprovider.h>
-#include <vala/valadatatype.h>
+#include <vala/valaclass.h>
+#include <vala/valatypesymbol.h>
 #include <gee/arraylist.h>
 #include <gee/collection.h>
 #include <vala/valaexpression.h>
-#include <vala/valatypereference.h>
+#include <vala/valadatatype.h>
 #include <gee/iterable.h>
 #include <gee/iterator.h>
 #include <vala/valacodenode.h>
@@ -34,19 +35,25 @@
 #include <vala/valasourcereference.h>
 #include <vala/valamethod.h>
 #include <vala/valamember.h>
+#include <vala/valaclasstype.h>
 #include <vala/valaformalparameter.h>
 #include <vala/valainvocationexpression.h>
+#include <vala/valavoidtype.h>
 #include <vala/valaassignment.h>
 #include <vala/valainterface.h>
 #include <vala/valascope.h>
 #include <gobject/valadbusmethod.h>
+#include <gobject/valaccodegenerator.h>
 #include <gobject/valadbussignal.h>
 
+
+
+
 struct _ValaDBusBindingProviderPrivate {
 	ValaCodeContext* _context;
-	ValaDataType* string_type;
-	ValaDataType* connection_type;
-	ValaDataType* dbus_error_type;
+	ValaClass* string_type;
+	ValaTypesymbol* connection_type;
+	ValaTypesymbol* dbus_error_type;
 	GeeCollection* symbols;
 };
 #define VALA_DBUS_BINDING_PROVIDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_DBUS_BINDING_PROVIDER, ValaDBusBindingProviderPrivate))
@@ -61,6 +68,7 @@ static ValaBindingProviderIface* vala_dbus_binding_provider_vala_binding_provide
 static void vala_dbus_binding_provider_dispose (GObject * obj);
 
 
+
 ValaDBusBindingProvider* vala_dbus_binding_provider_new (void) {
 	ValaDBusBindingProvider * self;
 	self = g_object_newv (VALA_TYPE_DBUS_BINDING_PROVIDER, 0, NULL);
@@ -71,17 +79,18 @@ ValaDBusBindingProvider* vala_dbus_binding_provider_new (void) {
 static ValaSymbol* vala_dbus_binding_provider_real_get_binding (ValaBindingProvider* base, ValaMemberAccess* ma) {
 	ValaDBusBindingProvider * self;
 	self = VALA_DBUS_BINDING_PROVIDER (base);
-	g_return_val_if_fail (VALA_IS_MEMBER_ACCESS (ma), NULL);
-	if (self->priv->connection_type != NULL && vala_member_access_get_inner (ma) != NULL && vala_expression_get_static_type (vala_member_access_get_inner (ma)) != NULL && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma))) == self->priv->connection_type) {
+	g_return_val_if_fail (ma == NULL || VALA_IS_MEMBER_ACCESS (ma), NULL);
+	if (self->priv->connection_type != NULL && vala_member_access_get_inner (ma) != NULL && vala_expression_get_static_type (vala_member_access_get_inner (ma)) != NULL && vala_data_type_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma))) == self->priv->connection_type) {
 		GeeCollection* type_args;
 		GeeIterator* type_args_it;
-		ValaTypeReference* ret_type;
-		ValaTypeReference* _tmp1;
+		ValaDataType* _tmp1;
+		ValaDataType* _tmp2;
+		ValaDataType* ret_type;
 		ValaMethod* m;
-		ValaTypeReference* string_type_ref;
-		ValaFormalParameter* _tmp3;
+		ValaClassType* string_type_ref;
 		ValaFormalParameter* _tmp4;
-		ValaMethod* _tmp5;
+		ValaFormalParameter* _tmp5;
+		ValaMethod* _tmp6;
 		type_args = vala_member_access_get_type_arguments (ma);
 		if (gee_collection_get_size (type_args) != 1) {
 			gpointer _tmp0;
@@ -89,91 +98,84 @@ static ValaSymbol* vala_dbus_binding_provider_real_get_binding (ValaBindingProvi
 		}
 		type_args_it = gee_iterable_iterator (GEE_ITERABLE (type_args));
 		gee_iterator_next (type_args_it);
-		ret_type = vala_type_reference_new ();
 		_tmp1 = NULL;
-		vala_type_reference_set_data_type (ret_type, vala_type_reference_get_data_type ((_tmp1 = gee_iterator_get (type_args_it))));
-		(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
-		if (!vala_dbus_binding_provider_is_dbus_interface (self, vala_type_reference_get_data_type (ret_type))) {
-			gpointer _tmp2;
-			return (_tmp2 = NULL, (type_args == NULL ? NULL : (type_args = (g_object_unref (type_args), NULL))), (type_args_it == NULL ? NULL : (type_args_it = (g_object_unref (type_args_it), NULL))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), _tmp2);
+		_tmp2 = NULL;
+		ret_type = (_tmp2 = vala_data_type_copy ((_tmp1 = ((ValaDataType*) gee_iterator_get (type_args_it)))), (_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL))), _tmp2);
+		if (!vala_dbus_binding_provider_is_dbus_interface (self, ret_type)) {
+			gpointer _tmp3;
+			return (_tmp3 = NULL, (type_args == NULL ? NULL : (type_args = (g_object_unref (type_args), NULL))), (type_args_it == NULL ? NULL : (type_args_it = (g_object_unref (type_args_it), NULL))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), _tmp3);
 		}
 		m = vala_code_context_create_method (self->priv->_context, "get_object", ret_type, vala_code_node_get_source_reference (VALA_CODE_NODE (ma)));
 		vala_method_set_cname (m, "dbus_g_proxy_new_for_name");
 		vala_member_add_cheader_filename (VALA_MEMBER (m), "dbus/dbus-glib.h");
 		vala_symbol_set_access (VALA_SYMBOL (m), VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
-		string_type_ref = vala_type_reference_new ();
-		vala_type_reference_set_data_type (string_type_ref, self->priv->string_type);
-		_tmp3 = NULL;
-		vala_method_add_parameter (m, (_tmp3 = vala_code_context_create_formal_parameter (self->priv->_context, "name", string_type_ref, NULL)));
-		(_tmp3 = (g_object_unref (_tmp3), NULL));
+		string_type_ref = vala_class_type_new (self->priv->string_type);
 		_tmp4 = NULL;
-		vala_method_add_parameter (m, (_tmp4 = vala_code_context_create_formal_parameter (self->priv->_context, "path", string_type_ref, NULL)));
-		(_tmp4 = (g_object_unref (_tmp4), NULL));
-		gee_collection_add (self->priv->symbols, m);
+		vala_method_add_parameter (m, (_tmp4 = vala_code_context_create_formal_parameter (self->priv->_context, "name", VALA_DATA_TYPE (string_type_ref), NULL)));
+		(_tmp4 == NULL ? NULL : (_tmp4 = (g_object_unref (_tmp4), NULL)));
 		_tmp5 = NULL;
-		return VALA_SYMBOL ((_tmp5 = m, (type_args == NULL ? NULL : (type_args = (g_object_unref (type_args), NULL))), (type_args_it == NULL ? NULL : (type_args_it = (g_object_unref (type_args_it), NULL))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), (string_type_ref == NULL ? NULL : (string_type_ref = (g_object_unref (string_type_ref), NULL))), _tmp5));
-		(type_args == NULL ? NULL : (type_args = (g_object_unref (type_args), NULL)));
-		(type_args_it == NULL ? NULL : (type_args_it = (g_object_unref (type_args_it), NULL)));
-		(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
-		(m = (g_object_unref (m), NULL));
-		(string_type_ref == NULL ? NULL : (string_type_ref = (g_object_unref (string_type_ref), NULL)));
+		vala_method_add_parameter (m, (_tmp5 = vala_code_context_create_formal_parameter (self->priv->_context, "path", VALA_DATA_TYPE (string_type_ref), NULL)));
+		(_tmp5 == NULL ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL)));
+		gee_collection_add (self->priv->symbols, m);
+		_tmp6 = NULL;
+		return VALA_SYMBOL ((_tmp6 = m, (type_args == NULL ? NULL : (type_args = (g_object_unref (type_args), NULL))), (type_args_it == NULL ? NULL : (type_args_it = (g_object_unref (type_args_it), NULL))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), (string_type_ref == NULL ? NULL : (string_type_ref = (g_object_unref (string_type_ref), NULL))), _tmp6));
 	} else {
-		if (vala_member_access_get_inner (ma) != NULL && vala_expression_get_static_type (vala_member_access_get_inner (ma)) != NULL && vala_dbus_binding_provider_is_dbus_interface (self, vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (ma))))) {
+		if (vala_member_access_get_inner (ma) != NULL && vala_expression_get_static_type (vala_member_access_get_inner (ma)) != NULL && vala_dbus_binding_provider_is_dbus_interface (self, vala_expression_get_static_type (vala_member_access_get_inner (ma)))) {
 			if (VALA_IS_INVOCATION_EXPRESSION (vala_code_node_get_parent_node (VALA_CODE_NODE (ma)))) {
-				ValaInvocationExpression* _tmp6;
+				ValaInvocationExpression* _tmp7;
 				ValaInvocationExpression* expr;
-				ValaTypeReference* ret_type;
+				ValaDataType* ret_type;
 				ValaDBusMethod* m;
-				ValaFormalParameter* _tmp7;
-				ValaDBusMethod* _tmp8;
-				_tmp6 = NULL;
-				expr = (_tmp6 = VALA_INVOCATION_EXPRESSION (vala_code_node_get_parent_node (VALA_CODE_NODE (ma))), (_tmp6 == NULL ? NULL : g_object_ref (_tmp6)));
-				ret_type = vala_type_reference_new ();
+				ValaFormalParameter* _tmp10;
+				ValaDBusMethod* _tmp11;
+				_tmp7 = NULL;
+				expr = (_tmp7 = VALA_INVOCATION_EXPRESSION (vala_code_node_get_parent_node (VALA_CODE_NODE (ma))), (_tmp7 == NULL ? NULL : g_object_ref (_tmp7)));
+				ret_type = NULL;
 				if (vala_expression_get_expected_type (VALA_EXPRESSION (expr)) != NULL) {
-					vala_type_reference_set_data_type (ret_type, vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (expr))));
-					vala_type_reference_set_transfers_ownership (ret_type, vala_data_type_is_reference_type (vala_type_reference_get_data_type (ret_type)));
+					ValaDataType* _tmp8;
+					_tmp8 = NULL;
+					ret_type = (_tmp8 = vala_data_type_copy (vala_expression_get_expected_type (VALA_EXPRESSION (expr))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), _tmp8);
+					vala_data_type_set_transfers_ownership (ret_type, vala_data_type_is_reference_type_or_type_parameter (ret_type));
+				} else {
+					ValaDataType* _tmp9;
+					_tmp9 = NULL;
+					ret_type = (_tmp9 = VALA_DATA_TYPE (vala_void_type_new ()), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), _tmp9);
 				}
 				m = vala_dbus_method_new (vala_member_access_get_member_name (ma), ret_type, vala_code_node_get_source_reference (VALA_CODE_NODE (ma)));
 				if (vala_expression_get_expected_type (VALA_EXPRESSION (expr)) != NULL) {
-					ValaTypeReference* error_type;
-					error_type = vala_type_reference_new ();
-					vala_type_reference_set_data_type (error_type, self->priv->dbus_error_type);
+					ValaDataType* error_type;
+					error_type = vala_ccode_generator_get_data_type_for_symbol (self->priv->dbus_error_type);
 					vala_method_add_error_domain (VALA_METHOD (m), error_type);
 					(error_type == NULL ? NULL : (error_type = (g_object_unref (error_type), NULL)));
 				}
 				vala_symbol_set_access (VALA_SYMBOL (m), VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
-				_tmp7 = NULL;
-				vala_method_add_parameter (VALA_METHOD (m), (_tmp7 = vala_formal_parameter_new_with_ellipsis (NULL)));
-				(_tmp7 == NULL ? NULL : (_tmp7 = (g_object_unref (_tmp7), NULL)));
+				_tmp10 = NULL;
+				vala_method_add_parameter (VALA_METHOD (m), (_tmp10 = vala_formal_parameter_new_with_ellipsis (NULL)));
+				(_tmp10 == NULL ? NULL : (_tmp10 = (g_object_unref (_tmp10), NULL)));
 				gee_collection_add (self->priv->symbols, m);
-				_tmp8 = NULL;
-				return VALA_SYMBOL ((_tmp8 = m, (expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), _tmp8));
-				(expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL)));
-				(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
-				(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
+				_tmp11 = NULL;
+				return VALA_SYMBOL ((_tmp11 = m, (expr == NULL ? NULL : (expr = (g_object_unref (expr), NULL))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), _tmp11));
 			} else {
 				if (VALA_IS_ASSIGNMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (ma)))) {
-					ValaAssignment* _tmp9;
+					ValaAssignment* _tmp12;
 					ValaAssignment* a;
-					ValaTypeReference* _tmp11;
-					ValaDBusSignal* _tmp12;
+					ValaVoidType* _tmp14;
+					ValaDBusSignal* _tmp15;
 					ValaDBusSignal* s;
-					ValaDBusSignal* _tmp13;
-					_tmp9 = NULL;
-					a = (_tmp9 = VALA_ASSIGNMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (ma))), (_tmp9 == NULL ? NULL : g_object_ref (_tmp9)));
-					if (vala_assignment_get_left (a) != VALA_EXPRESSION (ma)) {
-						gpointer _tmp10;
-						return (_tmp10 = NULL, (a == NULL ? NULL : (a = (g_object_unref (a), NULL))), _tmp10);
-					}
-					_tmp11 = NULL;
+					ValaDBusSignal* _tmp16;
 					_tmp12 = NULL;
-					s = (_tmp12 = vala_dbus_signal_new (vala_member_access_get_member_name (ma), (_tmp11 = vala_type_reference_new ()), vala_code_node_get_source_reference (VALA_CODE_NODE (ma))), (_tmp11 == NULL ? NULL : (_tmp11 = (g_object_unref (_tmp11), NULL))), _tmp12);
+					a = (_tmp12 = VALA_ASSIGNMENT (vala_code_node_get_parent_node (VALA_CODE_NODE (ma))), (_tmp12 == NULL ? NULL : g_object_ref (_tmp12)));
+					if (vala_assignment_get_left (a) != VALA_EXPRESSION (ma)) {
+						gpointer _tmp13;
+						return (_tmp13 = NULL, (a == NULL ? NULL : (a = (g_object_unref (a), NULL))), _tmp13);
+					}
+					_tmp14 = NULL;
+					_tmp15 = NULL;
+					s = (_tmp15 = vala_dbus_signal_new (vala_member_access_get_member_name (ma), VALA_DATA_TYPE ((_tmp14 = vala_void_type_new ())), vala_code_node_get_source_reference (VALA_CODE_NODE (ma))), (_tmp14 == NULL ? NULL : (_tmp14 = (g_object_unref (_tmp14), NULL))), _tmp15);
 					vala_symbol_set_access (VALA_SYMBOL (s), VALA_SYMBOL_ACCESSIBILITY_PUBLIC);
 					gee_collection_add (self->priv->symbols, s);
-					_tmp13 = NULL;
-					return VALA_SYMBOL ((_tmp13 = s, (a == NULL ? NULL : (a = (g_object_unref (a), NULL))), _tmp13));
-					(a == NULL ? NULL : (a = (g_object_unref (a), NULL)));
-					(s == NULL ? NULL : (s = (g_object_unref (s), NULL)));
+					_tmp16 = NULL;
+					return VALA_SYMBOL ((_tmp16 = s, (a == NULL ? NULL : (a = (g_object_unref (a), NULL))), _tmp16));
 				}
 			}
 		}
@@ -186,34 +188,34 @@ static gboolean vala_dbus_binding_provider_is_dbus_interface (ValaDBusBindingPro
 	ValaAttribute* _tmp1;
 	gboolean _tmp2;
 	g_return_val_if_fail (VALA_IS_DBUS_BINDING_PROVIDER (self), FALSE);
-	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), FALSE);
-	if (!(VALA_IS_INTERFACE (t))) {
+	g_return_val_if_fail (t == NULL || VALA_IS_DATA_TYPE (t), FALSE);
+	if (!(VALA_IS_INTERFACE (vala_data_type_get_data_type (t)))) {
 		return FALSE;
 	}
 	_tmp1 = NULL;
-	return (_tmp2 = ((_tmp1 = vala_code_node_get_attribute (VALA_CODE_NODE (t), "DBusInterface")) != NULL), (_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL))), _tmp2);
+	return (_tmp2 = ((_tmp1 = vala_code_node_get_attribute (VALA_CODE_NODE (vala_data_type_get_data_type (t)), "DBusInterface")) != NULL), (_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL))), _tmp2);
 }
 
 
 void vala_dbus_binding_provider_set_context (ValaDBusBindingProvider* self, ValaCodeContext* value) {
 	ValaCodeContext* _tmp1;
 	ValaCodeContext* _tmp0;
-	ValaDataType* _tmp2;
+	ValaClass* _tmp2;
 	ValaSymbol* dbus_ns;
 	g_return_if_fail (VALA_IS_DBUS_BINDING_PROVIDER (self));
 	_tmp1 = NULL;
 	_tmp0 = NULL;
 	self->priv->_context = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_context == NULL ? NULL : (self->priv->_context = (g_object_unref (self->priv->_context), NULL))), _tmp1);
 	_tmp2 = NULL;
-	self->priv->string_type = (_tmp2 = VALA_DATA_TYPE (vala_scope_lookup (vala_symbol_get_scope (VALA_SYMBOL (vala_code_context_get_root (self->priv->_context))), "string")), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), _tmp2);
+	self->priv->string_type = (_tmp2 = VALA_CLASS (vala_scope_lookup (vala_symbol_get_scope (VALA_SYMBOL (vala_code_context_get_root (self->priv->_context))), "string")), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), _tmp2);
 	dbus_ns = vala_scope_lookup (vala_symbol_get_scope (VALA_SYMBOL (vala_code_context_get_root (self->priv->_context))), "DBus");
 	if (dbus_ns != NULL) {
-		ValaDataType* _tmp3;
-		ValaDataType* _tmp4;
+		ValaTypesymbol* _tmp3;
+		ValaTypesymbol* _tmp4;
 		_tmp3 = NULL;
-		self->priv->connection_type = (_tmp3 = VALA_DATA_TYPE (vala_scope_lookup (vala_symbol_get_scope (dbus_ns), "Connection")), (self->priv->connection_type == NULL ? NULL : (self->priv->connection_type = (g_object_unref (self->priv->connection_type), NULL))), _tmp3);
+		self->priv->connection_type = (_tmp3 = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (dbus_ns), "Connection")), (self->priv->connection_type == NULL ? NULL : (self->priv->connection_type = (g_object_unref (self->priv->connection_type), NULL))), _tmp3);
 		_tmp4 = NULL;
-		self->priv->dbus_error_type = (_tmp4 = VALA_DATA_TYPE (vala_scope_lookup (vala_symbol_get_scope (dbus_ns), "Error")), (self->priv->dbus_error_type == NULL ? NULL : (self->priv->dbus_error_type = (g_object_unref (self->priv->dbus_error_type), NULL))), _tmp4);
+		self->priv->dbus_error_type = (_tmp4 = VALA_TYPESYMBOL (vala_scope_lookup (vala_symbol_get_scope (dbus_ns), "Error")), (self->priv->dbus_error_type == NULL ? NULL : (self->priv->dbus_error_type = (g_object_unref (self->priv->dbus_error_type), NULL))), _tmp4);
 	}
 	(dbus_ns == NULL ? NULL : (dbus_ns = (g_object_unref (dbus_ns), NULL)));
 }
