@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,10 +20,12 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include "valaarraymovemethod.h"
-#include <vala/valasourcereference.h>
-#include <vala/valasymbol.h>
 #include <vala/valaarraymovemethod.h>
+#include <vala/valasymbol.h>
+#include <vala/valacodenode.h>
+
+
+
 
 enum  {
 	VALA_ARRAY_MOVE_METHOD_DUMMY_PROPERTY
@@ -31,13 +33,13 @@ enum  {
 static gpointer vala_array_move_method_parent_class = NULL;
 
 
+
 /**
  * Creates a new array move method.
  *
  * @return newly created method
  */
-ValaArrayMoveMethod* vala_array_move_method_new (ValaSourceReference* source_reference)
-{
+ValaArrayMoveMethod* vala_array_move_method_new (ValaSourceReference* source_reference) {
 	ValaArrayMoveMethod * self;
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_ARRAY_MOVE_METHOD, 0, NULL);
@@ -47,19 +49,16 @@ ValaArrayMoveMethod* vala_array_move_method_new (ValaSourceReference* source_ref
 }
 
 
-static void vala_array_move_method_class_init (ValaArrayMoveMethodClass * klass)
-{
+static void vala_array_move_method_class_init (ValaArrayMoveMethodClass * klass) {
 	vala_array_move_method_parent_class = g_type_class_peek_parent (klass);
 }
 
 
-static void vala_array_move_method_init (ValaArrayMoveMethod * self)
-{
+static void vala_array_move_method_init (ValaArrayMoveMethod * self) {
 }
 
 
-GType vala_array_move_method_get_type (void)
-{
+GType vala_array_move_method_get_type (void) {
 	static GType vala_array_move_method_type_id = 0;
 	if (G_UNLIKELY (vala_array_move_method_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaArrayMoveMethodClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_array_move_method_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaArrayMoveMethod), 0, (GInstanceInitFunc) vala_array_move_method_init };
