@@ -1,11 +1,11 @@
 /* valaattributeprocessor.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
+ * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,11 +18,11 @@
  *
  * Author:
  * 	Jürg Billeter <j@bitron.ch>
- *	Raffaele Sandrini <rasa@gmx.ch>
+ *	Raffaele Sandrini <raffaele@sandrini.ch>
  */
 
-#ifndef __VALAATTRIBUTEPROCESSOR_H__
-#define __VALAATTRIBUTEPROCESSOR_H__
+#ifndef __VALA_VALAATTRIBUTEPROCESSOR_H__
+#define __VALA_VALAATTRIBUTEPROCESSOR_H__
 
 #include <glib.h>
 #include <glib-object.h>
@@ -36,8 +36,10 @@
 #include <vala/valaenum.h>
 #include <vala/valamethod.h>
 #include <vala/valacreationmethod.h>
+#include <vala/valaformalparameter.h>
 #include <vala/valaproperty.h>
-#include <vala/valacallback.h>
+#include <vala/valadelegate.h>
+#include <vala/valaconstant.h>
 #include <vala/valafield.h>
 #include <vala/valasignal.h>
 
@@ -59,16 +61,18 @@ typedef struct _ValaAttributeProcessorPrivate ValaAttributeProcessorPrivate;
  * Code visitor processing attributes associated with code nodes.
  */
 struct _ValaAttributeProcessor {
-	ValaCodeVisitor parent;
+	ValaCodeVisitor parent_instance;
 	ValaAttributeProcessorPrivate * priv;
 };
 struct _ValaAttributeProcessorClass {
-	ValaCodeVisitorClass parent;
+	ValaCodeVisitorClass parent_class;
 };
 
 void vala_attribute_processor_process (ValaAttributeProcessor* self, ValaCodeContext* context);
+ValaAttributeProcessor* vala_attribute_processor_new (void);
 GType vala_attribute_processor_get_type (void);
 
+
 G_END_DECLS
 
 #endif
