@@ -1,6 +1,6 @@
 /* valacatchclause.vala
  *
- * Copyright (C) 2007  Jürg Billeter
+ * Copyright (C) 2007-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,17 +21,20 @@
  */
 
 #include <vala/valacatchclause.h>
-#include <vala/valatypereference.h>
 #include <vala/valablock.h>
 #include <vala/valavariabledeclarator.h>
+#include <vala/valadatatype.h>
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaCatchClausePrivate {
-	ValaTypeReference* _type_reference;
 	char* _variable_name;
 	ValaBlock* _body;
 	ValaVariableDeclarator* _variable_declarator;
+	ValaDataType* _data_type;
 };
 #define VALA_CATCH_CLAUSE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CATCH_CLAUSE, ValaCatchClausePrivate))
 enum  {
@@ -43,10 +46,12 @@ enum  {
 };
 static void vala_catch_clause_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_catch_clause_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
+static void vala_catch_clause_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
 static gpointer vala_catch_clause_parent_class = NULL;
 static void vala_catch_clause_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new catch clause.
  *
@@ -56,9 +61,9 @@ static void vala_catch_clause_dispose (GObject * obj);
  * @param source_reference reference to source code
  * @return                 newly created catch clause
  */
-ValaCatchClause* vala_catch_clause_new (ValaTypeReference* type_reference, const char* variable_name, ValaBlock* body, ValaSourceReference* source_reference) {
+ValaCatchClause* vala_catch_clause_new (ValaDataType* type_reference, const char* variable_name, ValaBlock* body, ValaSourceReference* source_reference) {
 	ValaCatchClause * self;
-	g_return_val_if_fail (type_reference == NULL || VALA_IS_TYPE_REFERENCE (type_reference), NULL);
+	g_return_val_if_fail (type_reference == NULL || VALA_IS_DATA_TYPE (type_reference), NULL);
 	g_return_val_if_fail (body == NULL || VALA_IS_BLOCK (body), NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_CATCH_CLAUSE, 0, NULL);
@@ -73,7 +78,7 @@ ValaCatchClause* vala_catch_clause_new (ValaTypeReference* type_reference, const
 static void vala_catch_clause_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCatchClause * self;
 	self = VALA_CATCH_CLAUSE (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_catch_clause (visitor, self);
 }
 
@@ -81,25 +86,41 @@ static void vala_catch_clause_real_accept (ValaCodeNode* base, ValaCodeVisitor*
 static void vala_catch_clause_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCatchClause * self;
 	self = VALA_CATCH_CLAUSE (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
-	vala_code_node_accept (VALA_CODE_NODE (vala_catch_clause_get_type_reference (self)), visitor);
-	vala_code_node_accept (VALA_CODE_NODE (vala_catch_clause_get_body (self)), visitor);
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
+	if (vala_catch_clause_get_type_reference (self) != NULL) {
+		vala_code_node_accept (VALA_CODE_NODE (vala_catch_clause_get_type_reference (self)), visitor);
+	}
+	vala_code_node_accept (VALA_CODE_NODE (self->priv->_body), visitor);
 }
 
 
-ValaTypeReference* vala_catch_clause_get_type_reference (ValaCatchClause* self) {
+static void vala_catch_clause_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type) {
+	ValaCatchClause * self;
+	self = VALA_CATCH_CLAUSE (base);
+	g_return_if_fail (old_type == NULL || VALA_IS_DATA_TYPE (old_type));
+	g_return_if_fail (new_type == NULL || VALA_IS_DATA_TYPE (new_type));
+	if (vala_catch_clause_get_type_reference (self) == old_type) {
+		vala_catch_clause_set_type_reference (self, new_type);
+	}
+}
+
+
+ValaDataType* vala_catch_clause_get_type_reference (ValaCatchClause* self) {
 	g_return_val_if_fail (VALA_IS_CATCH_CLAUSE (self), NULL);
-	return self->priv->_type_reference;
+	return self->priv->_data_type;
 }
 
 
-void vala_catch_clause_set_type_reference (ValaCatchClause* self, ValaTypeReference* value) {
-	ValaTypeReference* _tmp2;
-	ValaTypeReference* _tmp1;
+void vala_catch_clause_set_type_reference (ValaCatchClause* self, ValaDataType* value) {
+	ValaDataType* _tmp2;
+	ValaDataType* _tmp1;
 	g_return_if_fail (VALA_IS_CATCH_CLAUSE (self));
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_type_reference = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), _tmp2);
+	self->priv->_data_type = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_data_type == NULL ? NULL : (self->priv->_data_type = (g_object_unref (self->priv->_data_type), NULL))), _tmp2);
+	if (self->priv->_data_type != NULL) {
+		vala_code_node_set_parent_node (VALA_CODE_NODE (self->priv->_data_type), VALA_CODE_NODE (self));
+	}
 }
 
 
@@ -205,7 +226,8 @@ static void vala_catch_clause_class_init (ValaCatchClauseClass * klass) {
 	G_OBJECT_CLASS (klass)->dispose = vala_catch_clause_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_catch_clause_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_catch_clause_real_accept_children;
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CATCH_CLAUSE_TYPE_REFERENCE, g_param_spec_object ("type-reference", "type-reference", "type-reference", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_catch_clause_real_replace_type;
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CATCH_CLAUSE_TYPE_REFERENCE, g_param_spec_object ("type-reference", "type-reference", "type-reference", VALA_TYPE_DATA_TYPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CATCH_CLAUSE_VARIABLE_NAME, g_param_spec_string ("variable-name", "variable-name", "variable-name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CATCH_CLAUSE_BODY, g_param_spec_object ("body", "body", "body", VALA_TYPE_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CATCH_CLAUSE_VARIABLE_DECLARATOR, g_param_spec_object ("variable-declarator", "variable-declarator", "variable-declarator", VALA_TYPE_VARIABLE_DECLARATOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -220,10 +242,10 @@ static void vala_catch_clause_init (ValaCatchClause * self) {
 static void vala_catch_clause_dispose (GObject * obj) {
 	ValaCatchClause * self;
 	self = VALA_CATCH_CLAUSE (obj);
-	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
-	(self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL));
+	self->priv->_variable_name = (g_free (self->priv->_variable_name), NULL);
 	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
 	(self->priv->_variable_declarator == NULL ? NULL : (self->priv->_variable_declarator = (g_object_unref (self->priv->_variable_declarator), NULL)));
+	(self->priv->_data_type == NULL ? NULL : (self->priv->_data_type = (g_object_unref (self->priv->_data_type), NULL)));
 	G_OBJECT_CLASS (vala_catch_clause_parent_class)->dispose (obj);
 }
 
