@@ -24,6 +24,9 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaConditionalExpressionPrivate {
 	ValaExpression* _condition;
 	ValaExpression* _true_expression;
@@ -42,6 +45,7 @@ static gpointer vala_conditional_expression_parent_class = NULL;
 static void vala_conditional_expression_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new conditional expression.
  *
@@ -87,9 +91,9 @@ static void vala_conditional_expression_real_accept (ValaCodeNode* base, ValaCod
 	ValaConditionalExpression * self;
 	self = VALA_CONDITIONAL_EXPRESSION (base);
 	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
-	vala_code_node_accept (VALA_CODE_NODE (vala_conditional_expression_get_condition (self)), visitor);
-	vala_code_node_accept (VALA_CODE_NODE (vala_conditional_expression_get_true_expression (self)), visitor);
-	vala_code_node_accept (VALA_CODE_NODE (vala_conditional_expression_get_false_expression (self)), visitor);
+	vala_code_node_accept (VALA_CODE_NODE (self->priv->_condition), visitor);
+	vala_code_node_accept (VALA_CODE_NODE (self->priv->_true_expression), visitor);
+	vala_code_node_accept (VALA_CODE_NODE (self->priv->_false_expression), visitor);
 	vala_code_visitor_visit_conditional_expression (visitor, self);
 }
 
@@ -97,7 +101,7 @@ static void vala_conditional_expression_real_accept (ValaCodeNode* base, ValaCod
 static gboolean vala_conditional_expression_real_is_pure (ValaExpression* base) {
 	ValaConditionalExpression * self;
 	self = VALA_CONDITIONAL_EXPRESSION (base);
-	return vala_expression_is_pure (vala_conditional_expression_get_condition (self)) && vala_expression_is_pure (vala_conditional_expression_get_true_expression (self)) && vala_expression_is_pure (vala_conditional_expression_get_false_expression (self));
+	return vala_expression_is_pure (self->priv->_condition) && vala_expression_is_pure (self->priv->_true_expression) && vala_expression_is_pure (self->priv->_false_expression);
 }
 
 
