@@ -1,6 +1,6 @@
 /* valaconstructor.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,9 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaConstructorPrivate {
 	ValaBlock* _body;
 	ValaFormalParameter* _this_parameter;
@@ -44,6 +47,7 @@ static gpointer vala_constructor_parent_class = NULL;
 static void vala_constructor_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new constructor.
  *
@@ -62,7 +66,7 @@ ValaConstructor* vala_constructor_new (ValaSourceReference* source) {
 static void vala_constructor_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaConstructor * self;
 	self = VALA_CONSTRUCTOR (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_constructor (visitor, self);
 }
 
@@ -70,9 +74,9 @@ static void vala_constructor_real_accept (ValaCodeNode* base, ValaCodeVisitor* v
 static void vala_constructor_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaConstructor * self;
 	self = VALA_CONSTRUCTOR (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
-	if (vala_constructor_get_body (self) != NULL) {
-		vala_code_node_accept (VALA_CODE_NODE (vala_constructor_get_body (self)), visitor);
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
+	if (self->priv->_body != NULL) {
+		vala_code_node_accept (VALA_CODE_NODE (self->priv->_body), visitor);
 	}
 }
 
@@ -171,7 +175,7 @@ static void vala_constructor_class_init (ValaConstructorClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_constructor_real_accept_children;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTRUCTOR_BODY, g_param_spec_object ("body", "body", "body", VALA_TYPE_BLOCK, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTRUCTOR_THIS_PARAMETER, g_param_spec_object ("this-parameter", "this-parameter", "this-parameter", VALA_TYPE_FORMAL_PARAMETER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTRUCTOR_INSTANCE, g_param_spec_boolean ("instance", "instance", "instance", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTRUCTOR_INSTANCE, g_param_spec_boolean ("instance", "instance", "instance", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
 
