@@ -1,6 +1,6 @@
 /* valacreationmethod.vala
  *
- * Copyright (C) 2007  Raffaele Sandrini, Jürg Billeter
+ * Copyright (C) 2007-2008  Raffaele Sandrini, Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,18 +17,23 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
  *
  * Author:
- *	Raffaele Sandrini <rasa@gmx.ch>
+ *	Raffaele Sandrini <raffaele@sandrini.ch>
  * 	Jürg Billeter <j@bitron.ch>
  */
 
 #include <vala/valacreationmethod.h>
 #include <vala/valasourcereference.h>
+#include <vala/valavoidtype.h>
 #include <vala/valasymbol.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valaformalparameter.h>
-#include <vala/valatypereference.h>
-#include <vala/valablock.h>
 #include <vala/valadatatype.h>
+#include <vala/valablock.h>
+#include <vala/valatypesymbol.h>
+#include <vala/valastruct.h>
+
+
+
 
 struct _ValaCreationMethodPrivate {
 	char* _type_name;
@@ -47,6 +52,7 @@ static gpointer vala_creation_method_parent_class = NULL;
 static void vala_creation_method_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new method.
  *
@@ -56,8 +62,12 @@ static void vala_creation_method_dispose (GObject * obj);
  */
 ValaCreationMethod* vala_creation_method_new (const char* type_name, const char* name, ValaSourceReference* source_reference) {
 	ValaCreationMethod * self;
+	ValaVoidType* _tmp0;
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_CREATION_METHOD, 0, NULL);
+	_tmp0 = NULL;
+	vala_method_set_return_type (VALA_METHOD (self), VALA_DATA_TYPE ((_tmp0 = vala_void_type_new ())));
+	(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
 	vala_creation_method_set_type_name (self, type_name);
 	vala_symbol_set_name (VALA_SYMBOL (self), name);
 	vala_code_node_set_source_reference (VALA_CODE_NODE (self), source_reference);
@@ -68,7 +78,7 @@ ValaCreationMethod* vala_creation_method_new (const char* type_name, const char*
 static void vala_creation_method_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCreationMethod * self;
 	self = VALA_CREATION_METHOD (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_creation_method (visitor, self);
 }
 
@@ -76,15 +86,15 @@ static void vala_creation_method_real_accept (ValaCodeNode* base, ValaCodeVisito
 static void vala_creation_method_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaCreationMethod * self;
 	self = VALA_CREATION_METHOD (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	{
 		GeeCollection* param_collection;
 		GeeIterator* param_it;
-		param_collection = vala_invokable_get_parameters (VALA_INVOKABLE (self));
+		param_collection = vala_method_get_parameters (VALA_METHOD (self));
 		param_it = gee_iterable_iterator (GEE_ITERABLE (param_collection));
 		while (gee_iterator_next (param_it)) {
 			ValaFormalParameter* param;
-			param = gee_iterator_get (param_it);
+			param = ((ValaFormalParameter*) gee_iterator_get (param_it));
 			{
 				vala_code_node_accept (VALA_CODE_NODE (param), visitor);
 				(param == NULL ? NULL : (param = (g_object_unref (param), NULL)));
@@ -99,8 +109,8 @@ static void vala_creation_method_real_accept_children (ValaCodeNode* base, ValaC
 		error_domain_collection = vala_method_get_error_domains (VALA_METHOD (self));
 		error_domain_it = gee_iterable_iterator (GEE_ITERABLE (error_domain_collection));
 		while (gee_iterator_next (error_domain_it)) {
-			ValaTypeReference* error_domain;
-			error_domain = gee_iterator_get (error_domain_it);
+			ValaDataType* error_domain;
+			error_domain = ((ValaDataType*) gee_iterator_get (error_domain_it));
 			{
 				vala_code_node_accept (VALA_CODE_NODE (error_domain), visitor);
 				(error_domain == NULL ? NULL : (error_domain = (g_object_unref (error_domain), NULL)));
@@ -117,30 +127,39 @@ static void vala_creation_method_real_accept_children (ValaCodeNode* base, ValaC
 
 static char* vala_creation_method_real_get_default_cname (ValaMethod* base) {
 	ValaCreationMethod * self;
+	ValaTypesymbol* _tmp1;
 	ValaSymbol* _tmp0;
-	ValaSymbol* parent;
+	ValaTypesymbol* parent;
+	char* infix;
 	self = VALA_CREATION_METHOD (base);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	parent = (_tmp0 = vala_symbol_get_parent_symbol (VALA_SYMBOL (self)), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
-	g_assert (VALA_IS_DATA_TYPE (parent));
-	if (vala_symbol_get_name (VALA_SYMBOL (self)) == NULL) {
-		char* _tmp1;
+	parent = (_tmp1 = (_tmp0 = vala_symbol_get_parent_symbol (VALA_SYMBOL (self)), (VALA_IS_TYPESYMBOL (_tmp0) ? ((ValaTypesymbol*) _tmp0) : NULL)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
+	infix = g_strdup ("new");
+	if (VALA_IS_STRUCT (parent)) {
 		char* _tmp2;
-		char* _tmp3;
-		_tmp1 = NULL;
 		_tmp2 = NULL;
-		_tmp3 = NULL;
-		return (_tmp3 = (_tmp2 = g_strdup_printf ("%snew", (_tmp1 = vala_symbol_get_lower_case_cprefix (VALA_SYMBOL ((VALA_DATA_TYPE (parent)))))), (_tmp1 = (g_free (_tmp1), NULL)), _tmp2), (parent == NULL ? NULL : (parent = (g_object_unref (parent), NULL))), _tmp3);
-	} else {
+		infix = (_tmp2 = g_strdup ("init"), (infix = (g_free (infix), NULL)), _tmp2);
+	}
+	if (g_utf8_strlen (vala_symbol_get_name (VALA_SYMBOL (self)), -1) == g_utf8_strlen (".new", -1)) {
+		char* _tmp3;
 		char* _tmp4;
 		char* _tmp5;
-		char* _tmp6;
+		_tmp3 = NULL;
 		_tmp4 = NULL;
 		_tmp5 = NULL;
+		return (_tmp5 = (_tmp4 = g_strdup_printf ("%s%s", (_tmp3 = vala_symbol_get_lower_case_cprefix (VALA_SYMBOL (parent))), infix), (_tmp3 = (g_free (_tmp3), NULL)), _tmp4), (parent == NULL ? NULL : (parent = (g_object_unref (parent), NULL))), (infix = (g_free (infix), NULL)), _tmp5);
+	} else {
+		char* _tmp6;
+		char* _tmp7;
+		char* _tmp8;
 		_tmp6 = NULL;
-		return (_tmp6 = (_tmp5 = g_strdup_printf ("%snew_%s", (_tmp4 = vala_symbol_get_lower_case_cprefix (VALA_SYMBOL ((VALA_DATA_TYPE (parent))))), vala_symbol_get_name (VALA_SYMBOL (self))), (_tmp4 = (g_free (_tmp4), NULL)), _tmp5), (parent == NULL ? NULL : (parent = (g_object_unref (parent), NULL))), _tmp6);
+		_tmp7 = NULL;
+		_tmp8 = NULL;
+		return (_tmp8 = (_tmp7 = g_strdup_printf ("%s%s_%s", (_tmp6 = vala_symbol_get_lower_case_cprefix (VALA_SYMBOL (parent))), infix, g_utf8_offset_to_pointer (vala_symbol_get_name (VALA_SYMBOL (self)), g_utf8_strlen (".new.", -1))), (_tmp6 = (g_free (_tmp6), NULL)), _tmp7), (parent == NULL ? NULL : (parent = (g_object_unref (parent), NULL))), (infix = (g_free (infix), NULL)), _tmp8);
 	}
 	(parent == NULL ? NULL : (parent = (g_object_unref (parent), NULL)));
+	infix = (g_free (infix), NULL);
 }
 
 
@@ -228,7 +247,7 @@ static void vala_creation_method_init (ValaCreationMethod * self) {
 static void vala_creation_method_dispose (GObject * obj) {
 	ValaCreationMethod * self;
 	self = VALA_CREATION_METHOD (obj);
-	(self->priv->_type_name = (g_free (self->priv->_type_name), NULL));
+	self->priv->_type_name = (g_free (self->priv->_type_name), NULL);
 	G_OBJECT_CLASS (vala_creation_method_parent_class)->dispose (obj);
 }
 
