@@ -24,7 +24,6 @@
 #include <vala/valadatatype.h>
 #include <gee/arraylist.h>
 #include <gee/readonlylist.h>
-#include <vala/valatypesymbol.h>
 #include <vala/valatypeparameter.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valareport.h>
@@ -35,6 +34,10 @@
 #include <vala/valapointertype.h>
 #include <vala/valaenum.h>
 #include <vala/valastruct.h>
+#include <vala/valasemanticanalyzer.h>
+
+
+
 
 struct _ValaDataTypePrivate {
 	gboolean _transfers_ownership;
@@ -65,7 +68,6 @@ static void vala_data_type_real_accept (ValaCodeNode* base, ValaCodeVisitor* vis
 static char* vala_data_type_real_get_cname (ValaDataType* self, gboolean var_type, gboolean const_type);
 static char* vala_data_type_real_get_lower_case_cname (ValaDataType* self, const char* infix);
 static char* vala_data_type_real_to_string (ValaCodeNode* base);
-static ValaDataType* vala_data_type_real_copy (ValaDataType* self);
 static gboolean vala_data_type_real_equals (ValaDataType* self, ValaDataType* type2);
 static gboolean vala_data_type_real_stricter (ValaDataType* self, ValaDataType* type2);
 static void vala_data_type_real_replace_type (ValaCodeNode* base, ValaDataType* old_type, ValaDataType* new_type);
@@ -76,6 +78,7 @@ static GeeCollection* vala_data_type_real_get_parameters (ValaDataType* self);
 static gboolean vala_data_type_real_is_reference_type_or_type_parameter (ValaDataType* self);
 static gboolean vala_data_type_real_is_array (ValaDataType* self);
 static GeeCollection* vala_data_type_real_get_symbols (ValaDataType* self);
+static ValaSymbol* vala_data_type_real_get_member (ValaDataType* self, const char* member_name);
 static ValaSymbol* vala_data_type_real_get_pointer_member (ValaDataType* self, const char* member_name);
 static gboolean vala_data_type_real_is_real_struct_type (ValaDataType* self);
 static char* vala_data_type_real_get_type_id (ValaDataType* self);
@@ -83,12 +86,6 @@ static gpointer vala_data_type_parent_class = NULL;
 static void vala_data_type_dispose (GObject * obj);
 
 
-ValaDataType* vala_data_type_new (void) {
-	ValaDataType * self;
-	self = g_object_newv (VALA_TYPE_DATA_TYPE, 0, NULL);
-	return self;
-}
-
 
 /**
  * Appends the specified type as generic type argument.
@@ -156,7 +153,7 @@ static void vala_data_type_real_accept (ValaCodeNode* base, ValaCodeVisitor* vis
 static char* vala_data_type_real_get_cname (ValaDataType* self, gboolean var_type, gboolean const_type) {
 	char* ptr;
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
-	if (vala_data_type_get_data_type (self) == NULL && vala_data_type_get_type_parameter (self) == NULL) {
+	if (self->priv->_data_type == NULL && self->priv->_type_parameter == NULL) {
 		if (var_type) {
 			return g_strdup ("gpointer");
 		} else {
@@ -164,12 +161,12 @@ static char* vala_data_type_real_get_cname (ValaDataType* self, gboolean var_typ
 		}
 	}
 	ptr = NULL;
-	if (vala_data_type_get_type_parameter (self) != NULL || (!vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self)) && !vala_data_type_get_is_ref (self) && !vala_data_type_get_is_out (self))) {
+	if (self->priv->_type_parameter != NULL || (!vala_typesymbol_is_reference_type (self->priv->_data_type) && !self->priv->_is_ref && !self->priv->_is_out)) {
 		char* _tmp2;
 		_tmp2 = NULL;
 		ptr = (_tmp2 = g_strdup (""), (ptr = (g_free (ptr), NULL)), _tmp2);
 	} else {
-		if ((vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self)) && !vala_data_type_get_is_ref (self) && !vala_data_type_get_is_out (self)) || (!vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self)) && (vala_data_type_get_is_ref (self) || vala_data_type_get_is_out (self)))) {
+		if ((vala_typesymbol_is_reference_type (self->priv->_data_type) && !self->priv->_is_ref && !self->priv->_is_out) || (!vala_typesymbol_is_reference_type (self->priv->_data_type) && (self->priv->_is_ref || self->priv->_is_out))) {
 			char* _tmp3;
 			_tmp3 = NULL;
 			ptr = (_tmp3 = g_strdup ("*"), (ptr = (g_free (ptr), NULL)), _tmp3);
@@ -179,16 +176,16 @@ static char* vala_data_type_real_get_cname (ValaDataType* self, gboolean var_typ
 			ptr = (_tmp4 = g_strdup ("**"), (ptr = (g_free (ptr), NULL)), _tmp4);
 		}
 	}
-	if (vala_data_type_get_data_type (self) != NULL) {
+	if (self->priv->_data_type != NULL) {
 		char* _tmp5;
 		char* _tmp6;
 		char* _tmp7;
 		_tmp5 = NULL;
 		_tmp6 = NULL;
 		_tmp7 = NULL;
-		return (_tmp7 = (_tmp6 = g_strconcat ((_tmp5 = vala_typesymbol_get_cname (vala_data_type_get_data_type (self), const_type)), ptr, NULL), (_tmp5 = (g_free (_tmp5), NULL)), _tmp6), (ptr = (g_free (ptr), NULL)), _tmp7);
+		return (_tmp7 = (_tmp6 = g_strconcat ((_tmp5 = vala_typesymbol_get_cname (self->priv->_data_type, const_type)), ptr, NULL), (_tmp5 = (g_free (_tmp5), NULL)), _tmp6), (ptr = (g_free (ptr), NULL)), _tmp7);
 	} else {
-		if (vala_data_type_get_type_parameter (self) != NULL) {
+		if (self->priv->_type_parameter != NULL) {
 			if (const_type) {
 				char* _tmp8;
 				_tmp8 = NULL;
@@ -235,13 +232,13 @@ char* vala_data_type_get_const_cname (ValaDataType* self) {
 		ValaTypesymbol* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		t = (_tmp1 = (_tmp0 = vala_data_type_get_data_type (vala_array_type_get_element_type ((VALA_ARRAY_TYPE (self)))), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (t == NULL ? NULL : (t = (g_object_unref (t), NULL))), _tmp1);
+		t = (_tmp1 = (_tmp0 = vala_array_type_get_element_type ((VALA_ARRAY_TYPE (self)))->priv->_data_type, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (t == NULL ? NULL : (t = (g_object_unref (t), NULL))), _tmp1);
 	} else {
 		ValaTypesymbol* _tmp3;
 		ValaTypesymbol* _tmp2;
 		_tmp3 = NULL;
 		_tmp2 = NULL;
-		t = (_tmp3 = (_tmp2 = vala_data_type_get_data_type (self), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (t == NULL ? NULL : (t = (g_object_unref (t), NULL))), _tmp3);
+		t = (_tmp3 = (_tmp2 = self->priv->_data_type, (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (t == NULL ? NULL : (t = (g_object_unref (t), NULL))), _tmp3);
 	}
 	if (!vala_typesymbol_is_reference_type (t)) {
 		char* _tmp4;
@@ -269,7 +266,7 @@ char* vala_data_type_get_const_cname (ValaDataType* self) {
  */
 static char* vala_data_type_real_get_lower_case_cname (ValaDataType* self, const char* infix) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
-	return vala_symbol_get_lower_case_cname (VALA_SYMBOL (vala_data_type_get_data_type (self)), infix);
+	return vala_symbol_get_lower_case_cname (VALA_SYMBOL (self->priv->_data_type), infix);
 }
 
 
@@ -285,17 +282,17 @@ static char* vala_data_type_real_to_string (ValaCodeNode* base) {
 	char* _tmp12;
 	self = VALA_DATA_TYPE (base);
 	s = NULL;
-	if (vala_data_type_get_data_type (self) != NULL) {
+	if (self->priv->_data_type != NULL) {
 		char* _tmp0;
 		_tmp0 = NULL;
-		s = (_tmp0 = vala_symbol_get_full_name (VALA_SYMBOL (vala_data_type_get_data_type (self))), (s = (g_free (s), NULL)), _tmp0);
+		s = (_tmp0 = vala_symbol_get_full_name (VALA_SYMBOL (self->priv->_data_type)), (s = (g_free (s), NULL)), _tmp0);
 	} else {
-		if (vala_data_type_get_type_parameter (self) != NULL) {
+		if (self->priv->_type_parameter != NULL) {
 			char* _tmp2;
 			const char* _tmp1;
 			_tmp2 = NULL;
 			_tmp1 = NULL;
-			s = (_tmp2 = (_tmp1 = vala_symbol_get_name (VALA_SYMBOL (vala_data_type_get_type_parameter (self))), (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (s = (g_free (s), NULL)), _tmp2);
+			s = (_tmp2 = (_tmp1 = vala_symbol_get_name (VALA_SYMBOL (self->priv->_type_parameter)), (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (s = (g_free (s), NULL)), _tmp2);
 		} else {
 			char* _tmp3;
 			_tmp3 = NULL;
@@ -326,22 +323,22 @@ static char* vala_data_type_real_to_string (ValaCodeNode* base) {
 					} else {
 						first = FALSE;
 					}
-					if (!vala_data_type_get_takes_ownership (type_arg)) {
+					if (!type_arg->priv->_takes_ownership) {
 						char* _tmp6;
 						_tmp6 = NULL;
 						s = (_tmp6 = g_strconcat (s, ("weak "), NULL), (s = (g_free (s), NULL)), _tmp6);
 					}
-					if (vala_data_type_get_data_type (type_arg) != NULL) {
+					if (type_arg->priv->_data_type != NULL) {
 						char* _tmp8;
 						char* _tmp7;
 						_tmp8 = NULL;
 						_tmp7 = NULL;
-						s = (_tmp8 = g_strconcat (s, (_tmp7 = (vala_symbol_get_full_name (VALA_SYMBOL (vala_data_type_get_data_type (type_arg))))), NULL), (s = (g_free (s), NULL)), _tmp8);
+						s = (_tmp8 = g_strconcat (s, (_tmp7 = (vala_symbol_get_full_name (VALA_SYMBOL (type_arg->priv->_data_type)))), NULL), (s = (g_free (s), NULL)), _tmp8);
 						_tmp7 = (g_free (_tmp7), NULL);
 					} else {
 						char* _tmp9;
 						_tmp9 = NULL;
-						s = (_tmp9 = g_strconcat (s, (vala_symbol_get_name (VALA_SYMBOL (vala_data_type_get_type_parameter (type_arg)))), NULL), (s = (g_free (s), NULL)), _tmp9);
+						s = (_tmp9 = g_strconcat (s, (vala_symbol_get_name (VALA_SYMBOL (type_arg->priv->_type_parameter))), NULL), (s = (g_free (s), NULL)), _tmp9);
 					}
 					(type_arg == NULL ? NULL : (type_arg = (g_object_unref (type_arg), NULL)));
 				}
@@ -351,7 +348,7 @@ static char* vala_data_type_real_to_string (ValaCodeNode* base) {
 		_tmp10 = NULL;
 		s = (_tmp10 = g_strconcat (s, (">"), NULL), (s = (g_free (s), NULL)), _tmp10);
 	}
-	if (vala_data_type_get_nullable (self)) {
+	if (self->priv->_nullable) {
 		char* _tmp11;
 		_tmp11 = NULL;
 		s = (_tmp11 = g_strconcat (s, ("?"), NULL), (s = (g_free (s), NULL)), _tmp11);
@@ -366,42 +363,6 @@ static char* vala_data_type_real_to_string (ValaCodeNode* base) {
  *
  * @return copy of this type reference
  */
-static ValaDataType* vala_data_type_real_copy (ValaDataType* self) {
-	ValaDataType* result;
-	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
-	result = vala_data_type_new ();
-	vala_code_node_set_source_reference (VALA_CODE_NODE (result), vala_code_node_get_source_reference (VALA_CODE_NODE (self)));
-	vala_data_type_set_transfers_ownership (result, vala_data_type_get_transfers_ownership (self));
-	vala_data_type_set_takes_ownership (result, vala_data_type_get_takes_ownership (self));
-	vala_data_type_set_is_out (result, vala_data_type_get_is_out (self));
-	vala_data_type_set_nullable (result, vala_data_type_get_nullable (self));
-	vala_data_type_set_requires_null_check (result, vala_data_type_get_requires_null_check (self));
-	vala_data_type_set_data_type (result, vala_data_type_get_data_type (self));
-	vala_data_type_set_type_parameter (result, vala_data_type_get_type_parameter (self));
-	vala_data_type_set_floating_reference (result, vala_data_type_get_floating_reference (self));
-	vala_data_type_set_is_ref (result, vala_data_type_get_is_ref (self));
-	{
-		GeeList* arg_collection;
-		GeeIterator* arg_it;
-		arg_collection = self->priv->type_argument_list;
-		arg_it = gee_iterable_iterator (GEE_ITERABLE (arg_collection));
-		while (gee_iterator_next (arg_it)) {
-			ValaDataType* arg;
-			arg = ((ValaDataType*) gee_iterator_get (arg_it));
-			{
-				ValaDataType* _tmp0;
-				_tmp0 = NULL;
-				gee_collection_add (GEE_COLLECTION (result->priv->type_argument_list), (_tmp0 = vala_data_type_copy (arg)));
-				(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
-				(arg == NULL ? NULL : (arg = (g_object_unref (arg), NULL)));
-			}
-		}
-		(arg_it == NULL ? NULL : (arg_it = (g_object_unref (arg_it), NULL)));
-	}
-	return result;
-}
-
-
 ValaDataType* vala_data_type_copy (ValaDataType* self) {
 	return VALA_DATA_TYPE_GET_CLASS (self)->copy (self);
 }
@@ -418,33 +379,33 @@ ValaDataType* vala_data_type_copy (ValaDataType* self) {
 static gboolean vala_data_type_real_equals (ValaDataType* self, ValaDataType* type2) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), FALSE);
 	g_return_val_if_fail (type2 == NULL || VALA_IS_DATA_TYPE (type2), FALSE);
-	if (vala_data_type_get_transfers_ownership (type2) != vala_data_type_get_transfers_ownership (self)) {
+	if (type2->priv->_transfers_ownership != self->priv->_transfers_ownership) {
 		return FALSE;
 	}
-	if (vala_data_type_get_takes_ownership (type2) != vala_data_type_get_takes_ownership (self)) {
+	if (type2->priv->_takes_ownership != self->priv->_takes_ownership) {
 		return FALSE;
 	}
-	if (vala_data_type_get_is_ref (type2) != vala_data_type_get_is_ref (self)) {
+	if (type2->priv->_is_ref != self->priv->_is_ref) {
 		return FALSE;
 	}
-	if (vala_data_type_get_is_out (type2) != vala_data_type_get_is_out (self)) {
+	if (type2->priv->_is_out != self->priv->_is_out) {
 		return FALSE;
 	}
-	if (vala_data_type_get_nullable (type2) != vala_data_type_get_nullable (self)) {
+	if (type2->priv->_nullable != self->priv->_nullable) {
 		return FALSE;
 	}
-	if (vala_data_type_get_data_type (type2) != vala_data_type_get_data_type (self)) {
+	if (type2->priv->_data_type != self->priv->_data_type) {
 		return FALSE;
 	}
-	if (vala_data_type_get_type_parameter (type2) != NULL || vala_data_type_get_type_parameter (self) != NULL) {
-		if (vala_data_type_get_type_parameter (type2) == NULL || vala_data_type_get_type_parameter (self) == NULL) {
+	if (type2->priv->_type_parameter != NULL || self->priv->_type_parameter != NULL) {
+		if (type2->priv->_type_parameter == NULL || self->priv->_type_parameter == NULL) {
 			return FALSE;
 		}
-		if (!vala_typeparameter_equals (vala_data_type_get_type_parameter (type2), vala_data_type_get_type_parameter (self))) {
+		if (!vala_typeparameter_equals (type2->priv->_type_parameter, self->priv->_type_parameter)) {
 			return FALSE;
 		}
 	}
-	if (vala_data_type_get_floating_reference (type2) != vala_data_type_get_floating_reference (self)) {
+	if (type2->priv->_floating_reference != self->priv->_floating_reference) {
 		return FALSE;
 	}
 	return TRUE;
@@ -466,30 +427,30 @@ gboolean vala_data_type_equals (ValaDataType* self, ValaDataType* type2) {
 static gboolean vala_data_type_real_stricter (ValaDataType* self, ValaDataType* type2) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), FALSE);
 	g_return_val_if_fail (type2 == NULL || VALA_IS_DATA_TYPE (type2), FALSE);
-	if (vala_data_type_get_transfers_ownership (type2) != vala_data_type_get_transfers_ownership (self)) {
+	if (type2->priv->_transfers_ownership != self->priv->_transfers_ownership) {
 		return FALSE;
 	}
-	if (vala_data_type_get_takes_ownership (type2) != vala_data_type_get_takes_ownership (self)) {
+	if (type2->priv->_takes_ownership != self->priv->_takes_ownership) {
 		return FALSE;
 	}
-	if (vala_data_type_get_is_ref (type2) != vala_data_type_get_is_ref (self)) {
+	if (type2->priv->_is_ref != self->priv->_is_ref) {
 		return FALSE;
 	}
-	if (vala_data_type_get_is_out (type2) != vala_data_type_get_is_out (self)) {
+	if (type2->priv->_is_out != self->priv->_is_out) {
 		return FALSE;
 	}
-	if (!vala_data_type_get_nullable (type2) && vala_data_type_get_nullable (self)) {
+	if (!type2->priv->_nullable && self->priv->_nullable) {
 		return FALSE;
 	}
-	if (vala_data_type_get_data_type (type2) != vala_data_type_get_data_type (self)) {
+	if (type2->priv->_data_type != self->priv->_data_type) {
 		return FALSE;
 	}
 	/* FIXME: allow this type reference to refer to a
 	        subtype of the type type2 is referring to*/
-	if (vala_data_type_get_type_parameter (type2) != vala_data_type_get_type_parameter (self)) {
+	if (type2->priv->_type_parameter != self->priv->_type_parameter) {
 		return FALSE;
 	}
-	if (vala_data_type_get_floating_reference (type2) != vala_data_type_get_floating_reference (self)) {
+	if (type2->priv->_floating_reference != self->priv->_floating_reference) {
 		return FALSE;
 	}
 	return TRUE;
@@ -531,46 +492,46 @@ static gboolean vala_data_type_real_compatible (ValaDataType* self, ValaDataType
 		return vala_delegate_type_get_delegate_symbol ((VALA_DELEGATE_TYPE (target_type))) == vala_delegate_type_get_delegate_symbol ((VALA_DELEGATE_TYPE (self)));
 	}
 	/* only null is compatible to null */
-	if (!(VALA_IS_POINTER_TYPE (target_type)) && vala_data_type_get_data_type (target_type) == NULL && vala_data_type_get_type_parameter (target_type) == NULL) {
-		return (vala_data_type_get_data_type (self) == NULL && vala_data_type_get_type_parameter (self) == NULL);
+	if (!(VALA_IS_POINTER_TYPE (target_type)) && target_type->priv->_data_type == NULL && target_type->priv->_type_parameter == NULL) {
+		return (self->priv->_data_type == NULL && self->priv->_type_parameter == NULL);
 	}
 	_tmp2 = NULL;
-	if ((_tmp3 = VALA_IS_POINTER_TYPE (target_type) || (vala_data_type_get_data_type (target_type) != NULL && (_tmp2 = vala_code_node_get_attribute (VALA_CODE_NODE (vala_data_type_get_data_type (target_type)), "PointerType")) != NULL), (_tmp2 == NULL ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL))), _tmp3)) {
+	if ((_tmp3 = VALA_IS_POINTER_TYPE (target_type) || (target_type->priv->_data_type != NULL && (_tmp2 = vala_code_node_get_attribute (VALA_CODE_NODE (target_type->priv->_data_type), "PointerType")) != NULL), (_tmp2 == NULL ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL))), _tmp3)) {
 		ValaAttribute* _tmp4;
 		gboolean _tmp5;
 		/* any reference or array type or pointer type can be cast to a generic pointer */
 		_tmp4 = NULL;
-		if ((_tmp5 = vala_data_type_get_type_parameter (self) != NULL || (vala_data_type_get_data_type (self) != NULL && (vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self)) || VALA_IS_ARRAY_TYPE (self) || VALA_IS_DELEGATE_TYPE (self) || (_tmp4 = vala_code_node_get_attribute (VALA_CODE_NODE (vala_data_type_get_data_type (self)), "PointerType")) != NULL)), (_tmp4 == NULL ? NULL : (_tmp4 = (g_object_unref (_tmp4), NULL))), _tmp5)) {
+		if ((_tmp5 = self->priv->_type_parameter != NULL || (self->priv->_data_type != NULL && (vala_typesymbol_is_reference_type (self->priv->_data_type) || VALA_IS_DELEGATE_TYPE (self) || (_tmp4 = vala_code_node_get_attribute (VALA_CODE_NODE (self->priv->_data_type), "PointerType")) != NULL)), (_tmp4 == NULL ? NULL : (_tmp4 = (g_object_unref (_tmp4), NULL))), _tmp5)) {
 			return TRUE;
 		}
 		return FALSE;
 	}
 	/* temporarily ignore type parameters */
-	if (vala_data_type_get_type_parameter (target_type) != NULL) {
+	if (target_type->priv->_type_parameter != NULL) {
 		return TRUE;
 	}
 	if (VALA_IS_ARRAY_TYPE (self) != VALA_IS_ARRAY_TYPE (target_type)) {
 		return FALSE;
 	}
-	if (VALA_IS_ENUM (vala_data_type_get_data_type (self)) && VALA_IS_STRUCT (vala_data_type_get_data_type (target_type)) && vala_struct_is_integer_type ((VALA_STRUCT (vala_data_type_get_data_type (target_type))))) {
+	if (VALA_IS_ENUM (self->priv->_data_type) && VALA_IS_STRUCT (target_type->priv->_data_type) && vala_struct_is_integer_type ((VALA_STRUCT (target_type->priv->_data_type)))) {
 		return TRUE;
 	}
-	if (vala_data_type_get_data_type (self) == vala_data_type_get_data_type (target_type)) {
-		if (vala_data_type_get_requires_null_check (self) && !vala_data_type_get_nullable (target_type) && vala_data_type_get_data_type (self) != NULL && vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self))) {
+	if (self->priv->_data_type == target_type->priv->_data_type) {
+		if (self->priv->_requires_null_check && !target_type->priv->_nullable && self->priv->_data_type != NULL && vala_typesymbol_is_reference_type (self->priv->_data_type)) {
 			return !enable_non_null;
 		}
 		return TRUE;
 	}
 	/* incompatibility between null and non-null types*/
-	if (VALA_IS_STRUCT (vala_data_type_get_data_type (self)) && VALA_IS_STRUCT (vala_data_type_get_data_type (target_type))) {
+	if (VALA_IS_STRUCT (self->priv->_data_type) && VALA_IS_STRUCT (target_type->priv->_data_type)) {
 		ValaStruct* _tmp13;
 		ValaStruct* expr_struct;
 		ValaStruct* _tmp14;
 		ValaStruct* expect_struct;
 		_tmp13 = NULL;
-		expr_struct = (_tmp13 = VALA_STRUCT (vala_data_type_get_data_type (self)), (_tmp13 == NULL ? NULL : g_object_ref (_tmp13)));
+		expr_struct = (_tmp13 = VALA_STRUCT (self->priv->_data_type), (_tmp13 == NULL ? NULL : g_object_ref (_tmp13)));
 		_tmp14 = NULL;
-		expect_struct = (_tmp14 = VALA_STRUCT (vala_data_type_get_data_type (target_type)), (_tmp14 == NULL ? NULL : g_object_ref (_tmp14)));
+		expect_struct = (_tmp14 = VALA_STRUCT (target_type->priv->_data_type), (_tmp14 == NULL ? NULL : g_object_ref (_tmp14)));
 		/* integer types may be implicitly cast to floating point types */
 		if (vala_struct_is_integer_type (expr_struct) && vala_struct_is_floating_type (expect_struct)) {
 			gboolean _tmp15;
@@ -585,8 +546,8 @@ static gboolean vala_data_type_real_compatible (ValaDataType* self, ValaDataType
 		(expr_struct == NULL ? NULL : (expr_struct = (g_object_unref (expr_struct), NULL)));
 		(expect_struct == NULL ? NULL : (expect_struct = (g_object_unref (expect_struct), NULL)));
 	}
-	if (vala_data_type_get_data_type (self) != NULL && vala_data_type_get_data_type (target_type) != NULL && vala_typesymbol_is_subtype_of (vala_data_type_get_data_type (self), vala_data_type_get_data_type (target_type))) {
-		if (vala_data_type_get_requires_null_check (self) && !vala_data_type_get_nullable (target_type) && vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self))) {
+	if (self->priv->_data_type != NULL && target_type->priv->_data_type != NULL && vala_typesymbol_is_subtype_of (self->priv->_data_type, target_type->priv->_data_type)) {
+		if (self->priv->_requires_null_check && !target_type->priv->_nullable && vala_typesymbol_is_reference_type (self->priv->_data_type)) {
 			return !enable_non_null;
 		}
 		return TRUE;
@@ -651,7 +612,7 @@ GeeCollection* vala_data_type_get_parameters (ValaDataType* self) {
 
 static gboolean vala_data_type_real_is_reference_type_or_type_parameter (ValaDataType* self) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), FALSE);
-	return (vala_data_type_get_data_type (self) != NULL && vala_typesymbol_is_reference_type (vala_data_type_get_data_type (self))) || vala_data_type_get_type_parameter (self) != NULL;
+	return (self->priv->_data_type != NULL && vala_typesymbol_is_reference_type (self->priv->_data_type)) || self->priv->_type_parameter != NULL;
 }
 
 
@@ -680,8 +641,8 @@ static GeeCollection* vala_data_type_real_get_symbols (ValaDataType* self) {
 	GeeArrayList* symbols;
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
 	symbols = gee_array_list_new (VALA_TYPE_SYMBOL, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal);
-	if (vala_data_type_get_data_type (self) != NULL) {
-		gee_collection_add (GEE_COLLECTION (symbols), vala_data_type_get_data_type (self));
+	if (self->priv->_data_type != NULL) {
+		gee_collection_add (GEE_COLLECTION (symbols), self->priv->_data_type);
 	}
 	return GEE_COLLECTION (symbols);
 }
@@ -692,6 +653,20 @@ GeeCollection* vala_data_type_get_symbols (ValaDataType* self) {
 }
 
 
+static ValaSymbol* vala_data_type_real_get_member (ValaDataType* self, const char* member_name) {
+	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
+	if (self->priv->_data_type != NULL) {
+		return vala_semantic_analyzer_symbol_lookup_inherited (VALA_SYMBOL (self->priv->_data_type), member_name);
+	}
+	return NULL;
+}
+
+
+ValaSymbol* vala_data_type_get_member (ValaDataType* self, const char* member_name) {
+	return VALA_DATA_TYPE_GET_CLASS (self)->get_member (self, member_name);
+}
+
+
 static ValaSymbol* vala_data_type_real_get_pointer_member (ValaDataType* self, const char* member_name) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
 	return NULL;
@@ -715,7 +690,7 @@ static gboolean vala_data_type_real_is_real_struct_type (ValaDataType* self) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), FALSE);
 	_tmp1 = NULL;
 	_tmp0 = NULL;
-	s = (_tmp1 = (_tmp0 = vala_data_type_get_data_type (self), (VALA_IS_STRUCT (_tmp0) ? ((ValaStruct*) _tmp0) : NULL)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
+	s = (_tmp1 = (_tmp0 = self->priv->_data_type, (VALA_IS_STRUCT (_tmp0) ? ((ValaStruct*) _tmp0) : NULL)), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
 	if (s != NULL && !vala_struct_is_simple_type (s)) {
 		gboolean _tmp2;
 		return (_tmp2 = TRUE, (s == NULL ? NULL : (s = (g_object_unref (s), NULL))), _tmp2);
@@ -731,8 +706,8 @@ gboolean vala_data_type_is_real_struct_type (ValaDataType* self) {
 
 static char* vala_data_type_real_get_type_id (ValaDataType* self) {
 	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
-	if (vala_data_type_get_data_type (self) != NULL) {
-		return vala_typesymbol_get_type_id (vala_data_type_get_data_type (self));
+	if (self->priv->_data_type != NULL) {
+		return vala_typesymbol_get_type_id (self->priv->_data_type);
 	} else {
 		return NULL;
 	}
@@ -942,7 +917,6 @@ static void vala_data_type_class_init (ValaDataTypeClass * klass) {
 	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_data_type_real_get_cname;
 	VALA_DATA_TYPE_CLASS (klass)->get_lower_case_cname = vala_data_type_real_get_lower_case_cname;
 	VALA_CODE_NODE_CLASS (klass)->to_string = vala_data_type_real_to_string;
-	VALA_DATA_TYPE_CLASS (klass)->copy = vala_data_type_real_copy;
 	VALA_DATA_TYPE_CLASS (klass)->equals = vala_data_type_real_equals;
 	VALA_DATA_TYPE_CLASS (klass)->stricter = vala_data_type_real_stricter;
 	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_data_type_real_replace_type;
@@ -953,6 +927,7 @@ static void vala_data_type_class_init (ValaDataTypeClass * klass) {
 	VALA_DATA_TYPE_CLASS (klass)->is_reference_type_or_type_parameter = vala_data_type_real_is_reference_type_or_type_parameter;
 	VALA_DATA_TYPE_CLASS (klass)->is_array = vala_data_type_real_is_array;
 	VALA_DATA_TYPE_CLASS (klass)->get_symbols = vala_data_type_real_get_symbols;
+	VALA_DATA_TYPE_CLASS (klass)->get_member = vala_data_type_real_get_member;
 	VALA_DATA_TYPE_CLASS (klass)->get_pointer_member = vala_data_type_real_get_pointer_member;
 	VALA_DATA_TYPE_CLASS (klass)->is_real_struct_type = vala_data_type_real_is_real_struct_type;
 	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_data_type_real_get_type_id;
@@ -987,7 +962,7 @@ GType vala_data_type_get_type (void) {
 	static GType vala_data_type_type_id = 0;
 	if (G_UNLIKELY (vala_data_type_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaDataTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_data_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaDataType), 0, (GInstanceInitFunc) vala_data_type_init };
-		vala_data_type_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaDataType", &g_define_type_info, 0);
+		vala_data_type_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaDataType", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
 	}
 	return vala_data_type_type_id;
 }
