@@ -44,11 +44,11 @@ typedef struct _ValaDestructorPrivate ValaDestructorPrivate;
  * Represents a class or instance destructor.
  */
 struct _ValaDestructor {
-	ValaSymbol parent;
+	ValaSymbol parent_instance;
 	ValaDestructorPrivate * priv;
 };
 struct _ValaDestructorClass {
-	ValaSymbolClass parent;
+	ValaSymbolClass parent_class;
 };
 
 ValaDestructor* vala_destructor_new (ValaSourceReference* source_reference);
@@ -58,6 +58,7 @@ gboolean vala_destructor_get_instance (ValaDestructor* self);
 void vala_destructor_set_instance (ValaDestructor* self, gboolean value);
 GType vala_destructor_get_type (void);
 
+
 G_END_DECLS
 
 #endif
