@@ -1,6 +1,6 @@
 /* valaenum.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,44 +32,51 @@
 #include <vala/valacreationmethod.h>
 #include <vala/valareport.h>
 #include <vala/valaformalparameter.h>
-#include <vala/valatypereference.h>
+#include <vala/valavaluetype.h>
+#include <vala/valadatatype.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valaattribute.h>
 
+
+
+
 struct _ValaEnumPrivate {
 	gboolean _is_flags;
-	gboolean _error_domain;
+	gboolean _has_type_id;
 	GeeList* values;
 	GeeList* methods;
 	char* cname;
 	char* cprefix;
 	char* lower_case_cprefix;
 	char* lower_case_csuffix;
+	char* type_id;
 };
 #define VALA_ENUM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ENUM, ValaEnumPrivate))
 enum  {
 	VALA_ENUM_DUMMY_PROPERTY,
 	VALA_ENUM_IS_FLAGS,
-	VALA_ENUM_ERROR_DOMAIN
+	VALA_ENUM_HAS_TYPE_ID
 };
 static void vala_enum_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_enum_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
-static char* vala_enum_real_get_cname (ValaDataType* base, gboolean const_type);
+static char* vala_enum_real_get_cname (ValaTypesymbol* base, gboolean const_type);
 static char* vala_enum_real_get_lower_case_cprefix (ValaSymbol* base);
 static char* vala_enum_get_lower_case_csuffix (ValaEnum* self);
 static char* vala_enum_real_get_lower_case_cname (ValaSymbol* base, const char* infix);
-static char* vala_enum_real_get_upper_case_cname (ValaDataType* base, const char* infix);
-static gboolean vala_enum_real_is_reference_type (ValaDataType* base);
+static char* vala_enum_real_get_upper_case_cname (ValaTypesymbol* base, const char* infix);
+static gboolean vala_enum_real_is_reference_type (ValaTypesymbol* base);
 static void vala_enum_set_cname (ValaEnum* self, const char* cname);
 static void vala_enum_process_ccode_attribute (ValaEnum* self, ValaAttribute* a);
-static char* vala_enum_real_get_type_id (ValaDataType* base);
-static char* vala_enum_real_get_marshaller_type_name (ValaDataType* base);
-static char* vala_enum_real_get_get_value_function (ValaDataType* base);
-static char* vala_enum_real_get_set_value_function (ValaDataType* base);
-static char* vala_enum_real_get_default_value (ValaDataType* base);
+static char* vala_enum_real_get_type_id (ValaTypesymbol* base);
+static char* vala_enum_real_get_marshaller_type_name (ValaTypesymbol* base);
+static char* vala_enum_real_get_get_value_function (ValaTypesymbol* base);
+static char* vala_enum_real_get_set_value_function (ValaTypesymbol* base);
+static char* vala_enum_real_get_default_value (ValaTypesymbol* base);
 static gpointer vala_enum_parent_class = NULL;
 static void vala_enum_dispose (GObject * obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static int _vala_strcmp0 (const char * str1, const char * str2);
+
 
 
 /**
@@ -81,7 +88,6 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
  */
 ValaEnum* vala_enum_new (const char* name, ValaSourceReference* source_reference) {
 	ValaEnum * self;
-	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_ENUM, 0, NULL);
 	vala_symbol_set_name (VALA_SYMBOL (self), name);
@@ -97,7 +103,7 @@ ValaEnum* vala_enum_new (const char* name, ValaSourceReference* source_reference
  */
 void vala_enum_add_value (ValaEnum* self, ValaEnumValue* value) {
 	g_return_if_fail (VALA_IS_ENUM (self));
-	g_return_if_fail (VALA_IS_ENUM_VALUE (value));
+	g_return_if_fail (value == NULL || VALA_IS_ENUM_VALUE (value));
 	gee_collection_add (GEE_COLLECTION (self->priv->values), value);
 	vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (self)), vala_symbol_get_name (VALA_SYMBOL (value)), VALA_SYMBOL (value));
 }
@@ -110,7 +116,7 @@ void vala_enum_add_value (ValaEnum* self, ValaEnumValue* value) {
  */
 void vala_enum_add_method (ValaEnum* self, ValaMethod* m) {
 	g_return_if_fail (VALA_IS_ENUM (self));
-	g_return_if_fail (VALA_IS_METHOD (m));
+	g_return_if_fail (m == NULL || VALA_IS_METHOD (m));
 	if (VALA_IS_CREATION_METHOD (m)) {
 		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), "construction methods may only be declared within classes and structs");
 		vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
@@ -118,13 +124,12 @@ void vala_enum_add_method (ValaEnum* self, ValaMethod* m) {
 	}
 	if (vala_method_get_instance (m)) {
 		ValaFormalParameter* _tmp1;
-		ValaTypeReference* _tmp0;
+		ValaValueType* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", (_tmp0 = vala_type_reference_new ()), NULL)));
+		vala_method_set_this_parameter (m, (_tmp1 = vala_formal_parameter_new ("this", VALA_DATA_TYPE ((_tmp0 = vala_value_type_new (VALA_TYPESYMBOL (self)))), NULL)));
 		(_tmp1 == NULL ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)));
 		(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
-		vala_type_reference_set_data_type (vala_formal_parameter_get_type_reference (vala_method_get_this_parameter (m)), VALA_DATA_TYPE (self));
 		vala_scope_add (vala_symbol_get_scope (VALA_SYMBOL (m)), vala_symbol_get_name (VALA_SYMBOL (vala_method_get_this_parameter (m))), VALA_SYMBOL (vala_method_get_this_parameter (m)));
 	}
 	gee_collection_add (GEE_COLLECTION (self->priv->methods), m);
@@ -157,7 +162,7 @@ GeeCollection* vala_enum_get_methods (ValaEnum* self) {
 static void vala_enum_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaEnum * self;
 	self = VALA_ENUM (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_enum (visitor, self);
 }
 
@@ -165,7 +170,7 @@ static void vala_enum_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
 static void vala_enum_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaEnum * self;
 	self = VALA_ENUM (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	{
 		GeeList* value_collection;
 		GeeIterator* value_it;
@@ -173,7 +178,7 @@ static void vala_enum_real_accept_children (ValaCodeNode* base, ValaCodeVisitor*
 		value_it = gee_iterable_iterator (GEE_ITERABLE (value_collection));
 		while (gee_iterator_next (value_it)) {
 			ValaEnumValue* value;
-			value = gee_iterator_get (value_it);
+			value = ((ValaEnumValue*) gee_iterator_get (value_it));
 			{
 				vala_code_node_accept (VALA_CODE_NODE (value), visitor);
 				(value == NULL ? NULL : (value = (g_object_unref (value), NULL)));
@@ -188,7 +193,7 @@ static void vala_enum_real_accept_children (ValaCodeNode* base, ValaCodeVisitor*
 		m_it = gee_iterable_iterator (GEE_ITERABLE (m_collection));
 		while (gee_iterator_next (m_it)) {
 			ValaMethod* m;
-			m = gee_iterator_get (m_it);
+			m = ((ValaMethod*) gee_iterator_get (m_it));
 			{
 				vala_code_node_accept (VALA_CODE_NODE (m), visitor);
 				(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
@@ -199,7 +204,7 @@ static void vala_enum_real_accept_children (ValaCodeNode* base, ValaCodeVisitor*
 }
 
 
-static char* vala_enum_real_get_cname (ValaDataType* base, gboolean const_type) {
+static char* vala_enum_real_get_cname (ValaTypesymbol* base, gboolean const_type) {
 	ValaEnum * self;
 	const char* _tmp2;
 	self = VALA_ENUM (base);
@@ -209,7 +214,7 @@ static char* vala_enum_real_get_cname (ValaDataType* base, gboolean const_type)
 		_tmp1 = NULL;
 		_tmp0 = NULL;
 		self->priv->cname = (_tmp1 = g_strdup_printf ("%s%s", (_tmp0 = vala_symbol_get_cprefix (vala_symbol_get_parent_symbol (VALA_SYMBOL (self)))), vala_symbol_get_name (VALA_SYMBOL (self))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
-		(_tmp0 = (g_free (_tmp0), NULL));
+		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp2 = NULL;
 	return (_tmp2 = self->priv->cname, (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
@@ -226,7 +231,7 @@ static char* vala_enum_real_get_lower_case_cprefix (ValaSymbol* base) {
 		_tmp1 = NULL;
 		_tmp0 = NULL;
 		self->priv->lower_case_cprefix = (_tmp1 = g_strdup_printf ("%s_", (_tmp0 = vala_symbol_get_lower_case_cname (VALA_SYMBOL (self), NULL))), (self->priv->lower_case_cprefix = (g_free (self->priv->lower_case_cprefix), NULL)), _tmp1);
-		(_tmp0 = (g_free (_tmp0), NULL));
+		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp2 = NULL;
 	return (_tmp2 = self->priv->lower_case_cprefix, (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
@@ -262,18 +267,18 @@ static char* vala_enum_real_get_lower_case_cname (ValaSymbol* base, const char*
 }
 
 
-static char* vala_enum_real_get_upper_case_cname (ValaDataType* base, const char* infix) {
+static char* vala_enum_real_get_upper_case_cname (ValaTypesymbol* base, const char* infix) {
 	ValaEnum * self;
 	char* _tmp0;
 	char* _tmp1;
 	self = VALA_ENUM (base);
 	_tmp0 = NULL;
 	_tmp1 = NULL;
-	return (_tmp1 = g_utf8_strup ((_tmp0 = vala_symbol_get_lower_case_cname (VALA_SYMBOL (self), NULL)), -1), (_tmp0 = (g_free (_tmp0), NULL)), _tmp1);
+	return (_tmp1 = g_utf8_strup ((_tmp0 = vala_symbol_get_lower_case_cname (VALA_SYMBOL (self), infix)), -1), (_tmp0 = (g_free (_tmp0), NULL)), _tmp1);
 }
 
 
-static gboolean vala_enum_real_is_reference_type (ValaDataType* base) {
+static gboolean vala_enum_real_is_reference_type (ValaTypesymbol* base) {
 	ValaEnum * self;
 	self = VALA_ENUM (base);
 	return FALSE;
@@ -281,11 +286,12 @@ static gboolean vala_enum_real_is_reference_type (ValaDataType* base) {
 
 
 static void vala_enum_set_cname (ValaEnum* self, const char* cname) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_ENUM (self));
-	g_return_if_fail (cname != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cname = (_tmp0 = g_strdup (cname), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp0);
+	self->priv->cname = (_tmp1 = (_tmp0 = cname, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cname = (g_free (self->priv->cname), NULL)), _tmp1);
 }
 
 
@@ -303,8 +309,8 @@ char* vala_enum_get_cprefix (ValaEnum* self) {
 		char* _tmp0;
 		_tmp1 = NULL;
 		_tmp0 = NULL;
-		self->priv->cprefix = (_tmp1 = g_strdup_printf ("%s_", (_tmp0 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (self), NULL))), (self->priv->cprefix = (g_free (self->priv->cprefix), NULL)), _tmp1);
-		(_tmp0 = (g_free (_tmp0), NULL));
+		self->priv->cprefix = (_tmp1 = g_strdup_printf ("%s_", (_tmp0 = vala_typesymbol_get_upper_case_cname (VALA_TYPESYMBOL (self), NULL))), (self->priv->cprefix = (g_free (self->priv->cprefix), NULL)), _tmp1);
+		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp2 = NULL;
 	return (_tmp2 = self->priv->cprefix, (_tmp2 == NULL ? NULL : g_strdup (_tmp2)));
@@ -318,28 +324,29 @@ char* vala_enum_get_cprefix (ValaEnum* self) {
  * @param cprefix the prefix to be used in C code
  */
 void vala_enum_set_cprefix (ValaEnum* self, const char* cprefix) {
-	char* _tmp0;
+	char* _tmp1;
+	const char* _tmp0;
 	g_return_if_fail (VALA_IS_ENUM (self));
-	g_return_if_fail (cprefix != NULL);
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	self->priv->cprefix = (_tmp0 = g_strdup (cprefix), (self->priv->cprefix = (g_free (self->priv->cprefix), NULL)), _tmp0);
+	self->priv->cprefix = (_tmp1 = (_tmp0 = cprefix, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->cprefix = (g_free (self->priv->cprefix), NULL)), _tmp1);
 }
 
 
 static void vala_enum_process_ccode_attribute (ValaEnum* self, ValaAttribute* a) {
 	g_return_if_fail (VALA_IS_ENUM (self));
-	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
+	g_return_if_fail (a == NULL || VALA_IS_ATTRIBUTE (a));
 	if (vala_attribute_has_argument (a, "cname")) {
 		char* _tmp0;
 		_tmp0 = NULL;
 		vala_enum_set_cname (self, (_tmp0 = vala_attribute_get_string (a, "cname")));
-		(_tmp0 = (g_free (_tmp0), NULL));
+		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	if (vala_attribute_has_argument (a, "cprefix")) {
 		char* _tmp1;
 		_tmp1 = NULL;
 		vala_enum_set_cprefix (self, (_tmp1 = vala_attribute_get_string (a, "cprefix")));
-		(_tmp1 = (g_free (_tmp1), NULL));
+		_tmp1 = (g_free (_tmp1), NULL);
 	}
 	if (vala_attribute_has_argument (a, "lower_case_csuffix")) {
 		char* _tmp2;
@@ -361,13 +368,21 @@ static void vala_enum_process_ccode_attribute (ValaEnum* self, ValaAttribute* a)
 				_tmp3 = NULL;
 				filename = (_tmp3 = *filename_it, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
 				{
-					vala_data_type_add_cheader_filename (VALA_DATA_TYPE (self), filename);
-					(filename = (g_free (filename), NULL));
+					vala_typesymbol_add_cheader_filename (VALA_TYPESYMBOL (self), filename);
+					filename = (g_free (filename), NULL);
 				}
 			}
-			(filename_collection = (_vala_array_free (filename_collection, filename_collection_length1, ((GDestroyNotify) g_free)), NULL));
+			filename_collection = (_vala_array_free (filename_collection, filename_collection_length1, ((GDestroyNotify) g_free)), NULL);
 		}
-		(val = (g_free (val), NULL));
+		val = (g_free (val), NULL);
+	}
+	if (vala_attribute_has_argument (a, "has_type_id")) {
+		vala_enum_set_has_type_id (self, vala_attribute_get_bool (a, "has_type_id"));
+	}
+	if (vala_attribute_has_argument (a, "type_id")) {
+		char* _tmp4;
+		_tmp4 = NULL;
+		self->priv->type_id = (_tmp4 = vala_attribute_get_string (a, "type_id"), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp4);
 	}
 }
 
@@ -385,17 +400,13 @@ void vala_enum_process_attributes (ValaEnum* self) {
 			ValaAttribute* _tmp0;
 			ValaAttribute* a;
 			_tmp0 = NULL;
-			a = (_tmp0 = a_it->data, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
+			a = (_tmp0 = ((ValaAttribute*) a_it->data), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
 			{
-				if (g_utf8_collate (vala_attribute_get_name (a), "CCode") == 0) {
+				if (_vala_strcmp0 (vala_attribute_get_name (a), "CCode") == 0) {
 					vala_enum_process_ccode_attribute (self, a);
 				} else {
-					if (g_utf8_collate (vala_attribute_get_name (a), "Flags") == 0) {
+					if (_vala_strcmp0 (vala_attribute_get_name (a), "Flags") == 0) {
 						vala_enum_set_is_flags (self, TRUE);
-					} else {
-						if (g_utf8_collate (vala_attribute_get_name (a), "ErrorDomain") == 0) {
-							vala_enum_set_error_domain (self, TRUE);
-						}
 					}
 				}
 				(a == NULL ? NULL : (a = (g_object_unref (a), NULL)));
@@ -405,39 +416,96 @@ void vala_enum_process_attributes (ValaEnum* self) {
 }
 
 
-static char* vala_enum_real_get_type_id (ValaDataType* base) {
-	ValaEnum * self;
-	self = VALA_ENUM (base);
-	return g_strdup ("G_TYPE_INT");
+void vala_enum_set_type_id (ValaEnum* self, const char* type_id) {
+	char* _tmp1;
+	const char* _tmp0;
+	g_return_if_fail (VALA_IS_ENUM (self));
+	_tmp1 = NULL;
+	_tmp0 = NULL;
+	self->priv->type_id = (_tmp1 = (_tmp0 = type_id, (_tmp0 == NULL ? NULL : g_strdup (_tmp0))), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp1);
 }
 
 
-/* FIXME: use GType-registered enums*/
-static char* vala_enum_real_get_marshaller_type_name (ValaDataType* base) {
+static char* vala_enum_real_get_type_id (ValaTypesymbol* base) {
 	ValaEnum * self;
+	const char* _tmp3;
 	self = VALA_ENUM (base);
-	return g_strdup ("INT");
+	if (self->priv->type_id == NULL) {
+		if (self->priv->_has_type_id) {
+			char* _tmp0;
+			_tmp0 = NULL;
+			self->priv->type_id = (_tmp0 = vala_typesymbol_get_upper_case_cname (VALA_TYPESYMBOL (self), "TYPE_"), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp0);
+		} else {
+			char* _tmp2;
+			const char* _tmp1;
+			_tmp2 = NULL;
+			_tmp1 = NULL;
+			self->priv->type_id = (_tmp2 = (_tmp1 = (self->priv->_is_flags ? "G_TYPE_UINT" : "G_TYPE_INT"), (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->type_id = (g_free (self->priv->type_id), NULL)), _tmp2);
+		}
+	}
+	_tmp3 = NULL;
+	return (_tmp3 = self->priv->type_id, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
 }
 
 
-/* FIXME: use GType-registered enums*/
-static char* vala_enum_real_get_get_value_function (ValaDataType* base) {
+static char* vala_enum_real_get_marshaller_type_name (ValaTypesymbol* base) {
 	ValaEnum * self;
 	self = VALA_ENUM (base);
-	return g_strdup ("g_value_get_int");
+	if (self->priv->_has_type_id) {
+		if (self->priv->_is_flags) {
+			return g_strdup ("FLAGS");
+		} else {
+			return g_strdup ("ENUM");
+		}
+	} else {
+		if (self->priv->_is_flags) {
+			return g_strdup ("UINT");
+		} else {
+			return g_strdup ("INT");
+		}
+	}
 }
 
 
-/* FIXME: use GType-registered enums*/
-static char* vala_enum_real_get_set_value_function (ValaDataType* base) {
+static char* vala_enum_real_get_get_value_function (ValaTypesymbol* base) {
 	ValaEnum * self;
 	self = VALA_ENUM (base);
-	return g_strdup ("g_value_set_int");
+	if (self->priv->_has_type_id) {
+		if (self->priv->_is_flags) {
+			return g_strdup ("g_value_get_flags");
+		} else {
+			return g_strdup ("g_value_get_enum");
+		}
+	} else {
+		if (self->priv->_is_flags) {
+			return g_strdup ("g_value_get_uint");
+		} else {
+			return g_strdup ("g_value_get_int");
+		}
+	}
 }
 
 
-/* FIXME: use GType-registered enums*/
-static char* vala_enum_real_get_default_value (ValaDataType* base) {
+static char* vala_enum_real_get_set_value_function (ValaTypesymbol* base) {
+	ValaEnum * self;
+	self = VALA_ENUM (base);
+	if (self->priv->_has_type_id) {
+		if (self->priv->_is_flags) {
+			return g_strdup ("g_value_set_flags");
+		} else {
+			return g_strdup ("g_value_set_enum");
+		}
+	} else {
+		if (self->priv->_is_flags) {
+			return g_strdup ("g_value_set_int");
+		} else {
+			return g_strdup ("g_value_set_uint");
+		}
+	}
+}
+
+
+static char* vala_enum_real_get_default_value (ValaTypesymbol* base) {
 	ValaEnum * self;
 	self = VALA_ENUM (base);
 	return g_strdup ("0");
@@ -456,15 +524,15 @@ void vala_enum_set_is_flags (ValaEnum* self, gboolean value) {
 }
 
 
-gboolean vala_enum_get_error_domain (ValaEnum* self) {
+gboolean vala_enum_get_has_type_id (ValaEnum* self) {
 	g_return_val_if_fail (VALA_IS_ENUM (self), FALSE);
-	return self->priv->_error_domain;
+	return self->priv->_has_type_id;
 }
 
 
-void vala_enum_set_error_domain (ValaEnum* self, gboolean value) {
+void vala_enum_set_has_type_id (ValaEnum* self, gboolean value) {
 	g_return_if_fail (VALA_IS_ENUM (self));
-	self->priv->_error_domain = value;
+	self->priv->_has_type_id = value;
 }
 
 
@@ -475,8 +543,8 @@ static void vala_enum_get_property (GObject * object, guint property_id, GValue
 		case VALA_ENUM_IS_FLAGS:
 		g_value_set_boolean (value, vala_enum_get_is_flags (self));
 		break;
-		case VALA_ENUM_ERROR_DOMAIN:
-		g_value_set_boolean (value, vala_enum_get_error_domain (self));
+		case VALA_ENUM_HAS_TYPE_ID:
+		g_value_set_boolean (value, vala_enum_get_has_type_id (self));
 		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -492,8 +560,8 @@ static void vala_enum_set_property (GObject * object, guint property_id, const G
 		case VALA_ENUM_IS_FLAGS:
 		vala_enum_set_is_flags (self, g_value_get_boolean (value));
 		break;
-		case VALA_ENUM_ERROR_DOMAIN:
-		vala_enum_set_error_domain (self, g_value_get_boolean (value));
+		case VALA_ENUM_HAS_TYPE_ID:
+		vala_enum_set_has_type_id (self, g_value_get_boolean (value));
 		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -510,23 +578,24 @@ static void vala_enum_class_init (ValaEnumClass * klass) {
 	G_OBJECT_CLASS (klass)->dispose = vala_enum_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_enum_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_enum_real_accept_children;
-	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_enum_real_get_cname;
+	VALA_TYPESYMBOL_CLASS (klass)->get_cname = vala_enum_real_get_cname;
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cprefix = vala_enum_real_get_lower_case_cprefix;
 	VALA_SYMBOL_CLASS (klass)->get_lower_case_cname = vala_enum_real_get_lower_case_cname;
-	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_enum_real_get_upper_case_cname;
-	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_enum_real_is_reference_type;
-	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_enum_real_get_type_id;
-	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_enum_real_get_marshaller_type_name;
-	VALA_DATA_TYPE_CLASS (klass)->get_get_value_function = vala_enum_real_get_get_value_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_set_value_function = vala_enum_real_get_set_value_function;
-	VALA_DATA_TYPE_CLASS (klass)->get_default_value = vala_enum_real_get_default_value;
+	VALA_TYPESYMBOL_CLASS (klass)->get_upper_case_cname = vala_enum_real_get_upper_case_cname;
+	VALA_TYPESYMBOL_CLASS (klass)->is_reference_type = vala_enum_real_is_reference_type;
+	VALA_TYPESYMBOL_CLASS (klass)->get_type_id = vala_enum_real_get_type_id;
+	VALA_TYPESYMBOL_CLASS (klass)->get_marshaller_type_name = vala_enum_real_get_marshaller_type_name;
+	VALA_TYPESYMBOL_CLASS (klass)->get_get_value_function = vala_enum_real_get_get_value_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_set_value_function = vala_enum_real_get_set_value_function;
+	VALA_TYPESYMBOL_CLASS (klass)->get_default_value = vala_enum_real_get_default_value;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ENUM_IS_FLAGS, g_param_spec_boolean ("is-flags", "is-flags", "is-flags", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ENUM_ERROR_DOMAIN, g_param_spec_boolean ("error-domain", "error-domain", "error-domain", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ENUM_HAS_TYPE_ID, g_param_spec_boolean ("has-type-id", "has-type-id", "has-type-id", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
 
 static void vala_enum_init (ValaEnum * self) {
 	self->priv = VALA_ENUM_GET_PRIVATE (self);
+	self->priv->_has_type_id = TRUE;
 	self->priv->values = GEE_LIST (gee_array_list_new (VALA_TYPE_ENUM_VALUE, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 	self->priv->methods = GEE_LIST (gee_array_list_new (VALA_TYPE_METHOD, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
 }
@@ -537,10 +606,11 @@ static void vala_enum_dispose (GObject * obj) {
 	self = VALA_ENUM (obj);
 	(self->priv->values == NULL ? NULL : (self->priv->values = (g_object_unref (self->priv->values), NULL)));
 	(self->priv->methods == NULL ? NULL : (self->priv->methods = (g_object_unref (self->priv->methods), NULL)));
-	(self->priv->cname = (g_free (self->priv->cname), NULL));
-	(self->priv->cprefix = (g_free (self->priv->cprefix), NULL));
-	(self->priv->lower_case_cprefix = (g_free (self->priv->lower_case_cprefix), NULL));
-	(self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL));
+	self->priv->cname = (g_free (self->priv->cname), NULL);
+	self->priv->cprefix = (g_free (self->priv->cprefix), NULL);
+	self->priv->lower_case_cprefix = (g_free (self->priv->lower_case_cprefix), NULL);
+	self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL);
+	self->priv->type_id = (g_free (self->priv->type_id), NULL);
 	G_OBJECT_CLASS (vala_enum_parent_class)->dispose (obj);
 }
 
@@ -549,7 +619,7 @@ GType vala_enum_get_type (void) {
 	static GType vala_enum_type_id = 0;
 	if (G_UNLIKELY (vala_enum_type_id == 0)) {
 		static const GTypeInfo g_define_type_info = { sizeof (ValaEnumClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_enum_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaEnum), 0, (GInstanceInitFunc) vala_enum_init };
-		vala_enum_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaEnum", &g_define_type_info, 0);
+		vala_enum_type_id = g_type_register_static (VALA_TYPE_TYPESYMBOL, "ValaEnum", &g_define_type_info, 0);
 	}
 	return vala_enum_type_id;
 }
@@ -572,5 +642,16 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
 }
 
 
+static int _vala_strcmp0 (const char * str1, const char * str2) {
+	if (str1 == NULL) {
+		return -(str1 != str2);
+	}
+	if (str2 == NULL) {
+		return (str1 != str2);
+	}
+	return strcmp (str1, str2);
+}
+
+
 
 
