@@ -26,6 +26,9 @@
 #include <vala/valacodevisitor.h>
 #include <vala/valaenum.h>
 
+
+
+
 struct _ValaEnumValuePrivate {
 	ValaExpression* _value;
 	char* cname;
@@ -41,6 +44,7 @@ static gpointer vala_enum_value_parent_class = NULL;
 static void vala_enum_value_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new enum value.
  *
@@ -76,6 +80,41 @@ ValaEnumValue* vala_enum_value_new_with_value (const char* name, ValaExpression*
 }
 
 
+/**
+ * Returns the string literal of this signal to be used in C code.
+ * (FIXME: from vlaasignal.vala)
+ *
+ * @return string literal to be used in C code
+ */
+ValaCCodeConstant* vala_enum_value_get_canonical_cconstant (ValaEnumValue* self) {
+	GString* str;
+	const char* _tmp0;
+	char* i;
+	ValaCCodeConstant* _tmp3;
+	g_return_val_if_fail (VALA_IS_ENUM_VALUE (self), NULL);
+	str = g_string_new ("\"");
+	_tmp0 = NULL;
+	i = (_tmp0 = vala_symbol_get_name (VALA_SYMBOL (self)), (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
+	while (g_utf8_strlen (i, -1) > 0) {
+		gunichar c;
+		char* _tmp2;
+		const char* _tmp1;
+		c = g_utf8_get_char (i);
+		if (c == '_') {
+			g_string_append_c (str, '-');
+		} else {
+			g_string_append_unichar (str, g_unichar_tolower (c));
+		}
+		_tmp2 = NULL;
+		_tmp1 = NULL;
+		i = (_tmp2 = (_tmp1 = g_utf8_next_char (i), (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (i = (g_free (i), NULL)), _tmp2);
+	}
+	g_string_append_c (str, '"');
+	_tmp3 = NULL;
+	return (_tmp3 = vala_ccode_constant_new (str->str), (str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL))), (i = (g_free (i), NULL)), _tmp3);
+}
+
+
 static void vala_enum_value_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaEnumValue * self;
 	self = VALA_ENUM_VALUE (base);
@@ -88,8 +127,8 @@ static void vala_enum_value_real_accept_children (ValaCodeNode* base, ValaCodeVi
 	ValaEnumValue * self;
 	self = VALA_ENUM_VALUE (base);
 	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
-	if (vala_enum_value_get_value (self) != NULL) {
-		vala_code_node_accept (VALA_CODE_NODE (vala_enum_value_get_value (self)), visitor);
+	if (self->priv->_value != NULL) {
+		vala_code_node_accept (VALA_CODE_NODE (self->priv->_value), visitor);
 	}
 }
 
