@@ -1,6 +1,6 @@
 /* valaintegerliteral.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,9 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaIntegerLiteralPrivate {
 	char* _value;
 };
@@ -34,10 +37,12 @@ enum  {
 };
 static void vala_integer_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static char* vala_integer_literal_real_to_string (ValaCodeNode* base);
+static gboolean vala_integer_literal_real_is_pure (ValaExpression* base);
 static gpointer vala_integer_literal_parent_class = NULL;
 static void vala_integer_literal_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new integer literal.
  *
@@ -47,7 +52,6 @@ static void vala_integer_literal_dispose (GObject * obj);
  */
 ValaIntegerLiteral* vala_integer_literal_new (const char* i, ValaSourceReference* source) {
 	ValaIntegerLiteral * self;
-	g_return_val_if_fail (i != NULL, NULL);
 	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
 	self = g_object_newv (VALA_TYPE_INTEGER_LITERAL, 0, NULL);
 	vala_integer_literal_set_value (self, i);
@@ -59,15 +63,17 @@ ValaIntegerLiteral* vala_integer_literal_new (const char* i, ValaSourceReference
 static void vala_integer_literal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaIntegerLiteral * self;
 	self = VALA_INTEGER_LITERAL (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_integer_literal (visitor, self);
 }
 
 
 static char* vala_integer_literal_real_to_string (ValaCodeNode* base) {
 	ValaIntegerLiteral * self;
+	const char* _tmp0;
 	self = VALA_INTEGER_LITERAL (base);
-	return g_strdup (vala_integer_literal_get_value (self));
+	_tmp0 = NULL;
+	return (_tmp0 = self->priv->_value, (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
 }
 
 
@@ -77,25 +83,27 @@ static char* vala_integer_literal_real_to_string (ValaCodeNode* base) {
  * @return the name of literal type
  */
 char* vala_integer_literal_get_type_name (ValaIntegerLiteral* self) {
+	const char* _tmp0;
 	char* number;
 	gint l;
 	gboolean u;
 	gint64 n;
 	g_return_val_if_fail (VALA_IS_INTEGER_LITERAL (self), NULL);
-	number = g_strdup (vala_integer_literal_get_value (self));
+	_tmp0 = NULL;
+	number = (_tmp0 = self->priv->_value, (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
 	l = 0;
 	while (g_str_has_suffix (number, "L")) {
-		char* _tmp0;
+		char* _tmp1;
 		l++;
-		_tmp0 = NULL;
-		number = (_tmp0 = g_strndup (number, strlen (number) - 1), (number = (g_free (number), NULL)), _tmp0);
+		_tmp1 = NULL;
+		number = (_tmp1 = g_strndup (number, ((gulong) strlen (number) - 1)), (number = (g_free (number), NULL)), _tmp1);
 	}
 	u = FALSE;
 	if (g_str_has_suffix (number, "U")) {
-		char* _tmp1;
+		char* _tmp2;
 		u = TRUE;
-		_tmp1 = NULL;
-		number = (_tmp1 = g_strndup (number, strlen (number) - 1), (number = (g_free (number), NULL)), _tmp1);
+		_tmp2 = NULL;
+		number = (_tmp2 = g_strndup (number, ((gulong) strlen (number) - 1)), (number = (g_free (number), NULL)), _tmp2);
 	}
 	n = g_ascii_strtoll (number, NULL, 0);
 	if (!u && n > 0x7fffffff) {
@@ -109,38 +117,45 @@ char* vala_integer_literal_get_type_name (ValaIntegerLiteral* self) {
 	}
 	if (l == 0) {
 		if (u) {
-			char* _tmp2;
-			_tmp2 = NULL;
-			return (_tmp2 = g_strdup ("uint"), (number = (g_free (number), NULL)), _tmp2);
-		} else {
 			char* _tmp3;
 			_tmp3 = NULL;
-			return (_tmp3 = g_strdup ("int"), (number = (g_free (number), NULL)), _tmp3);
+			return (_tmp3 = g_strdup ("uint"), (number = (g_free (number), NULL)), _tmp3);
+		} else {
+			char* _tmp4;
+			_tmp4 = NULL;
+			return (_tmp4 = g_strdup ("int"), (number = (g_free (number), NULL)), _tmp4);
 		}
 	} else {
 		if (l == 1) {
 			if (u) {
-				char* _tmp4;
-				_tmp4 = NULL;
-				return (_tmp4 = g_strdup ("ulong"), (number = (g_free (number), NULL)), _tmp4);
-			} else {
 				char* _tmp5;
 				_tmp5 = NULL;
-				return (_tmp5 = g_strdup ("long"), (number = (g_free (number), NULL)), _tmp5);
+				return (_tmp5 = g_strdup ("ulong"), (number = (g_free (number), NULL)), _tmp5);
+			} else {
+				char* _tmp6;
+				_tmp6 = NULL;
+				return (_tmp6 = g_strdup ("long"), (number = (g_free (number), NULL)), _tmp6);
 			}
 		} else {
 			if (u) {
-				char* _tmp6;
-				_tmp6 = NULL;
-				return (_tmp6 = g_strdup ("uint64"), (number = (g_free (number), NULL)), _tmp6);
-			} else {
 				char* _tmp7;
 				_tmp7 = NULL;
-				return (_tmp7 = g_strdup ("int64"), (number = (g_free (number), NULL)), _tmp7);
+				return (_tmp7 = g_strdup ("uint64"), (number = (g_free (number), NULL)), _tmp7);
+			} else {
+				char* _tmp8;
+				_tmp8 = NULL;
+				return (_tmp8 = g_strdup ("int64"), (number = (g_free (number), NULL)), _tmp8);
 			}
 		}
 	}
-	(number = (g_free (number), NULL));
+	number = (g_free (number), NULL);
+}
+
+
+static gboolean vala_integer_literal_real_is_pure (ValaExpression* base) {
+	ValaIntegerLiteral * self;
+	self = VALA_INTEGER_LITERAL (base);
+	return TRUE;
 }
 
 
@@ -151,10 +166,12 @@ char* vala_integer_literal_get_value (ValaIntegerLiteral* self) {
 
 
 void vala_integer_literal_set_value (ValaIntegerLiteral* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_INTEGER_LITERAL (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_value = (_tmp1 = g_strdup (value), (self->priv->_value = (g_free (self->priv->_value), NULL)), _tmp1);
+	self->priv->_value = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_value = (g_free (self->priv->_value), NULL)), _tmp2);
 }
 
 
@@ -194,6 +211,7 @@ static void vala_integer_literal_class_init (ValaIntegerLiteralClass * klass) {
 	G_OBJECT_CLASS (klass)->dispose = vala_integer_literal_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_integer_literal_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->to_string = vala_integer_literal_real_to_string;
+	VALA_EXPRESSION_CLASS (klass)->is_pure = vala_integer_literal_real_is_pure;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_INTEGER_LITERAL_VALUE, g_param_spec_string ("value", "value", "value", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
@@ -206,7 +224,7 @@ static void vala_integer_literal_init (ValaIntegerLiteral * self) {
 static void vala_integer_literal_dispose (GObject * obj) {
 	ValaIntegerLiteral * self;
 	self = VALA_INTEGER_LITERAL (obj);
-	(self->priv->_value = (g_free (self->priv->_value), NULL));
+	self->priv->_value = (g_free (self->priv->_value), NULL);
 	G_OBJECT_CLASS (vala_integer_literal_parent_class)->dispose (obj);
 }
 
