@@ -1,11 +1,11 @@
 /* valainterface.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,15 +20,15 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#ifndef __VALAINTERFACE_H__
-#define __VALAINTERFACE_H__
+#ifndef __VALA_VALAINTERFACE_H__
+#define __VALA_VALAINTERFACE_H__
 
 #include <glib.h>
 #include <glib-object.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gee/collection.h>
-#include <vala/valadatatype.h>
+#include <vala/valatypesymbol.h>
 #include <vala/valacodenode.h>
 
 G_BEGIN_DECLS
@@ -47,18 +47,18 @@ typedef struct _ValaInterfacePrivate ValaInterfacePrivate;
  * Represents a class declaration in the source code.
  */
 struct _ValaInterface {
-	ValaDataType parent;
+	ValaTypesymbol parent_instance;
 	ValaInterfacePrivate * priv;
 };
 struct _ValaInterfaceClass {
-	ValaDataTypeClass parent;
+	ValaTypesymbolClass parent_class;
 };
 
 ValaInterface* vala_interface_new (const char* name, ValaSourceReference* source_reference);
 void vala_interface_add_type_parameter (ValaInterface* self, ValaTypeParameter* p);
 GeeCollection* vala_interface_get_type_parameters (ValaInterface* self);
-void vala_interface_add_prerequisite (ValaInterface* self, ValaTypeReference* type);
-void vala_interface_prepend_prerequisite (ValaInterface* self, ValaTypeReference* type);
+void vala_interface_add_prerequisite (ValaInterface* self, ValaDataType* type);
+void vala_interface_prepend_prerequisite (ValaInterface* self, ValaDataType* type);
 GeeCollection* vala_interface_get_prerequisites (ValaInterface* self);
 void vala_interface_add_method (ValaInterface* self, ValaMethod* m);
 GeeCollection* vala_interface_get_methods (ValaInterface* self);
@@ -68,15 +68,23 @@ void vala_interface_add_property (ValaInterface* self, ValaProperty* prop);
 GeeCollection* vala_interface_get_properties (ValaInterface* self);
 void vala_interface_add_signal (ValaInterface* self, ValaSignal* sig);
 GeeCollection* vala_interface_get_signals (ValaInterface* self);
+void vala_interface_add_class (ValaInterface* self, ValaClass* cl);
+void vala_interface_add_struct (ValaInterface* self, ValaStruct* st);
+void vala_interface_add_enum (ValaInterface* self, ValaEnum* en);
+void vala_interface_add_delegate (ValaInterface* self, ValaDelegate* d);
 char* vala_interface_get_lower_case_csuffix (ValaInterface* self);
+char* vala_interface_get_default_lower_case_csuffix (ValaInterface* self);
 void vala_interface_set_lower_case_csuffix (ValaInterface* self, const char* csuffix);
 void vala_interface_process_attributes (ValaInterface* self);
 char* vala_interface_get_type_cname (ValaInterface* self);
 void vala_interface_set_type_cname (ValaInterface* self, const char* type_cname);
 gboolean vala_interface_get_is_static (ValaInterface* self);
 void vala_interface_set_is_static (ValaInterface* self, gboolean value);
+gboolean vala_interface_get_declaration_only (ValaInterface* self);
+void vala_interface_set_declaration_only (ValaInterface* self, gboolean value);
 GType vala_interface_get_type (void);
 
+
 G_END_DECLS
 
 #endif
