@@ -18,7 +18,7 @@
  *
  * Author:
  * 	Jürg Billeter <j@bitron.ch>
- *	Raffaele Sandrini <rasa@gmx.ch>
+ *	Raffaele Sandrini <raffaele@sandrini.ch>
  */
 
 #ifndef __VALA_VALAINTERFACEWRITER_H__
@@ -35,7 +35,8 @@
 #include <vala/valastruct.h>
 #include <vala/valainterface.h>
 #include <vala/valaenum.h>
-#include <vala/valaenumvalue.h>
+#include <vala/valaerrordomain.h>
+#include <vala/valaerrorcode.h>
 #include <vala/valaconstant.h>
 #include <vala/valafield.h>
 #include <vala/valadelegate.h>
@@ -73,6 +74,7 @@ void vala_interface_writer_write_file (ValaInterfaceWriter* self, ValaCodeContex
 ValaInterfaceWriter* vala_interface_writer_new (void);
 GType vala_interface_writer_get_type (void);
 
+
 G_END_DECLS
 
 #endif
