@@ -26,6 +26,9 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaMemberInitializerPrivate {
 	char* _name;
 	ValaExpression* _initializer;
@@ -43,6 +46,7 @@ static gpointer vala_member_initializer_parent_class = NULL;
 static void vala_member_initializer_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new member initializer.
  *
@@ -67,7 +71,7 @@ static void vala_member_initializer_real_accept (ValaCodeNode* base, ValaCodeVis
 	ValaMemberInitializer * self;
 	self = VALA_MEMBER_INITIALIZER (base);
 	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
-	vala_code_node_accept (VALA_CODE_NODE (vala_member_initializer_get_initializer (self)), visitor);
+	vala_code_node_accept (VALA_CODE_NODE (self->priv->_initializer), visitor);
 }
 
 
