@@ -46,11 +46,11 @@ typedef struct _ValaNamedArgumentPrivate ValaNamedArgumentPrivate;
  * when creating objects and attributes where no parameter list exists.
  */
 struct _ValaNamedArgument {
-	ValaCodeNode parent;
+	ValaCodeNode parent_instance;
 	ValaNamedArgumentPrivate * priv;
 };
 struct _ValaNamedArgumentClass {
-	ValaCodeNodeClass parent;
+	ValaCodeNodeClass parent_class;
 };
 
 ValaNamedArgument* vala_named_argument_new (const char* _name, ValaExpression* arg, ValaSourceReference* source);
@@ -60,6 +60,7 @@ ValaExpression* vala_named_argument_get_argument (ValaNamedArgument* self);
 void vala_named_argument_set_argument (ValaNamedArgument* self, ValaExpression* value);
 GType vala_named_argument_get_type (void);
 
+
 G_END_DECLS
 
 #endif
