@@ -1,6 +1,6 @@
 /* valasourcefile.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,20 +24,24 @@
 #include <gee/arraylist.h>
 #include <gee/list.h>
 #include <gee/readonlycollection.h>
+#include <glib/gstdio.h>
 #include <vala/valasourcefilecycle.h>
 #include <vala/valacodecontext.h>
 #include <vala/valanamespacereference.h>
 #include <vala/valacodevisitor.h>
 #include <vala/valasymbol.h>
-#include <vala/valadatatype.h>
+#include <vala/valatypesymbol.h>
 #include <vala/valamethod.h>
 #include <vala/valafield.h>
 #include <vala/valaproperty.h>
 #include <vala/valaconstant.h>
 #include <vala/valaformalparameter.h>
-#include <vala/valatypereference.h>
+#include <vala/valadatatype.h>
 #include <vala/valasourcereference.h>
 
+
+
+
 struct _ValaSourceFilePrivate {
 	char* _filename;
 	char* _comment;
@@ -57,6 +61,7 @@ struct _ValaSourceFilePrivate {
 	GeeList* source_internal_includes;
 	GeeList* header_internal_full_dependencies;
 	GeeList* header_internal_dependencies;
+	GeeArrayList* source_array;
 };
 #define VALA_SOURCE_FILE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SOURCE_FILE, ValaSourceFilePrivate))
 enum  {
@@ -71,8 +76,12 @@ enum  {
 };
 static char* vala_source_file_get_subdir (ValaSourceFile* self);
 static char* vala_source_file_get_destination_directory (ValaSourceFile* self);
+static void vala_source_file_read_source_file (ValaSourceFile* self);
 static gpointer vala_source_file_parent_class = NULL;
 static void vala_source_file_dispose (GObject * obj);
+static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static int _vala_strcmp0 (const char * str1, const char * str2);
+
 
 
 /**
@@ -84,8 +93,7 @@ static void vala_source_file_dispose (GObject * obj);
  */
 ValaSourceFile* vala_source_file_new (ValaCodeContext* context, const char* filename, gboolean pkg) {
 	ValaSourceFile * self;
-	g_return_val_if_fail (VALA_IS_CODE_CONTEXT (context), NULL);
-	g_return_val_if_fail (filename != NULL, NULL);
+	g_return_val_if_fail (context == NULL || VALA_IS_CODE_CONTEXT (context), NULL);
 	self = g_object_newv (VALA_TYPE_SOURCE_FILE, 0, NULL);
 	vala_source_file_set_context (self, context);
 	vala_source_file_set_filename (self, filename);
@@ -101,7 +109,7 @@ ValaSourceFile* vala_source_file_new (ValaCodeContext* context, const char* file
  */
 void vala_source_file_add_using_directive (ValaSourceFile* self, ValaNamespaceReference* ns) {
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
-	g_return_if_fail (VALA_IS_NAMESPACE_REFERENCE (ns));
+	g_return_if_fail (ns == NULL || VALA_IS_NAMESPACE_REFERENCE (ns));
 	gee_collection_add (GEE_COLLECTION (self->priv->using_directives), ns);
 }
 
@@ -124,7 +132,7 @@ GeeCollection* vala_source_file_get_using_directives (ValaSourceFile* self) {
  */
 void vala_source_file_add_node (ValaSourceFile* self, ValaCodeNode* node) {
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
-	g_return_if_fail (VALA_IS_CODE_NODE (node));
+	g_return_if_fail (node == NULL || VALA_IS_CODE_NODE (node));
 	gee_collection_add (GEE_COLLECTION (self->priv->nodes), node);
 }
 
@@ -142,14 +150,14 @@ GeeCollection* vala_source_file_get_nodes (ValaSourceFile* self) {
 
 void vala_source_file_accept (ValaSourceFile* self, ValaCodeVisitor* visitor) {
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_source_file (visitor, self);
 }
 
 
 void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* visitor) {
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	{
 		GeeList* ns_ref_collection;
 		GeeIterator* ns_ref_it;
@@ -157,7 +165,7 @@ void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* vi
 		ns_ref_it = gee_iterable_iterator (GEE_ITERABLE (ns_ref_collection));
 		while (gee_iterator_next (ns_ref_it)) {
 			ValaNamespaceReference* ns_ref;
-			ns_ref = gee_iterator_get (ns_ref_it);
+			ns_ref = ((ValaNamespaceReference*) gee_iterator_get (ns_ref_it));
 			{
 				vala_code_node_accept (VALA_CODE_NODE (ns_ref), visitor);
 				(ns_ref == NULL ? NULL : (ns_ref = (g_object_unref (ns_ref), NULL)));
@@ -172,7 +180,7 @@ void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* vi
 		node_it = gee_iterable_iterator (GEE_ITERABLE (node_collection));
 		while (gee_iterator_next (node_it)) {
 			ValaCodeNode* node;
-			node = gee_iterator_get (node_it);
+			node = ((ValaCodeNode*) gee_iterator_get (node_it));
 			{
 				vala_code_node_accept (node, visitor);
 				(node == NULL ? NULL : (node = (g_object_unref (node), NULL)));
@@ -185,29 +193,27 @@ void vala_source_file_accept_children (ValaSourceFile* self, ValaCodeVisitor* vi
 
 static char* vala_source_file_get_subdir (ValaSourceFile* self) {
 	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
-	if (vala_code_context_get_basedir (vala_source_file_get_context (self)) == NULL) {
+	if (vala_code_context_get_basedir (self->priv->_context) == NULL) {
 		return g_strdup ("");
 	}
 	/* filename and basedir are already canonicalized*/
-	if (g_str_has_prefix (vala_source_file_get_filename (self), vala_code_context_get_basedir (vala_source_file_get_context (self)))) {
+	if (g_str_has_prefix (self->priv->_filename, vala_code_context_get_basedir (self->priv->_context))) {
 		char* basename;
 		char* _tmp1;
 		char* subdir;
 		char* _tmp4;
-		basename = g_path_get_basename (vala_source_file_get_filename (self));
+		basename = g_path_get_basename (self->priv->_filename);
 		_tmp1 = NULL;
-		subdir = (_tmp1 = g_utf8_offset_to_pointer (vala_source_file_get_filename (self), g_utf8_strlen (vala_code_context_get_basedir (vala_source_file_get_context (self)), -1)), g_strndup (_tmp1, g_utf8_offset_to_pointer (_tmp1, g_utf8_strlen (vala_source_file_get_filename (self), -1) - g_utf8_strlen (vala_code_context_get_basedir (vala_source_file_get_context (self)), -1) - g_utf8_strlen (basename, -1)) - _tmp1));
+		subdir = (_tmp1 = g_utf8_offset_to_pointer (self->priv->_filename, g_utf8_strlen (vala_code_context_get_basedir (self->priv->_context), -1)), g_strndup (_tmp1, g_utf8_offset_to_pointer (_tmp1, g_utf8_strlen (self->priv->_filename, -1) - g_utf8_strlen (vala_code_context_get_basedir (self->priv->_context), -1) - g_utf8_strlen (basename, -1)) - _tmp1));
 		while (g_utf8_get_char (g_utf8_offset_to_pointer (subdir, 0)) == '/') {
 			char* _tmp3;
 			const char* _tmp2;
 			_tmp3 = NULL;
 			_tmp2 = NULL;
-			subdir = (_tmp3 = (_tmp2 = g_utf8_offset_to_pointer (subdir, 1), (_tmp2 == NULL ? NULL : g_strdup (_tmp2))), (subdir = (g_free (subdir), NULL)), _tmp3);
+			subdir = (_tmp3 = (_tmp2 = g_utf8_offset_to_pointer (subdir, ((glong) 1)), (_tmp2 == NULL ? NULL : g_strdup (_tmp2))), (subdir = (g_free (subdir), NULL)), _tmp3);
 		}
 		_tmp4 = NULL;
 		return (_tmp4 = subdir, (basename = (g_free (basename), NULL)), _tmp4);
-		(basename = (g_free (basename), NULL));
-		(subdir = (g_free (subdir), NULL));
 	}
 	return g_strdup ("");
 }
@@ -217,12 +223,12 @@ static char* vala_source_file_get_destination_directory (ValaSourceFile* self) {
 	char* _tmp1;
 	char* _tmp2;
 	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
-	if (vala_code_context_get_directory (vala_source_file_get_context (self)) == NULL) {
+	if (vala_code_context_get_directory (self->priv->_context) == NULL) {
 		return vala_source_file_get_subdir (self);
 	}
 	_tmp1 = NULL;
 	_tmp2 = NULL;
-	return (_tmp2 = g_strdup_printf ("%s/%s", vala_code_context_get_directory (vala_source_file_get_context (self)), (_tmp1 = vala_source_file_get_subdir (self))), (_tmp1 = (g_free (_tmp1), NULL)), _tmp2);
+	return (_tmp2 = g_strdup_printf ("%s/%s", vala_code_context_get_directory (self->priv->_context), (_tmp1 = vala_source_file_get_subdir (self))), (_tmp1 = (g_free (_tmp1), NULL)), _tmp2);
 }
 
 
@@ -239,14 +245,14 @@ char* vala_source_file_get_cheader_filename (ValaSourceFile* self) {
 		char* _tmp0;
 		char* _tmp2;
 		char* _tmp1;
-		basename = g_strndup (vala_source_file_get_filename (self), ((guint) (g_utf8_strlen (vala_source_file_get_filename (self), -1) - g_utf8_strlen (".vala", -1))));
+		basename = g_strndup (self->priv->_filename, ((gulong) ((guint) (g_utf8_strlen (self->priv->_filename, -1) - g_utf8_strlen (".vala", -1)))));
 		_tmp0 = NULL;
 		basename = (_tmp0 = g_path_get_basename (basename), (basename = (g_free (basename), NULL)), _tmp0);
 		_tmp2 = NULL;
 		_tmp1 = NULL;
 		self->priv->cheader_filename = (_tmp2 = g_strdup_printf ("%s%s.h", (_tmp1 = vala_source_file_get_destination_directory (self)), basename), (self->priv->cheader_filename = (g_free (self->priv->cheader_filename), NULL)), _tmp2);
-		(_tmp1 = (g_free (_tmp1), NULL));
-		(basename = (g_free (basename), NULL));
+		_tmp1 = (g_free (_tmp1), NULL);
+		basename = (g_free (basename), NULL);
 	}
 	_tmp3 = NULL;
 	return (_tmp3 = self->priv->cheader_filename, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
@@ -266,14 +272,14 @@ char* vala_source_file_get_csource_filename (ValaSourceFile* self) {
 		char* _tmp0;
 		char* _tmp2;
 		char* _tmp1;
-		basename = g_strndup (vala_source_file_get_filename (self), ((guint) (g_utf8_strlen (vala_source_file_get_filename (self), -1) - g_utf8_strlen (".vala", -1))));
+		basename = g_strndup (self->priv->_filename, ((gulong) ((guint) (g_utf8_strlen (self->priv->_filename, -1) - g_utf8_strlen (".vala", -1)))));
 		_tmp0 = NULL;
 		basename = (_tmp0 = g_path_get_basename (basename), (basename = (g_free (basename), NULL)), _tmp0);
 		_tmp2 = NULL;
 		_tmp1 = NULL;
 		self->priv->csource_filename = (_tmp2 = g_strdup_printf ("%s%s.c", (_tmp1 = vala_source_file_get_destination_directory (self)), basename), (self->priv->csource_filename = (g_free (self->priv->csource_filename), NULL)), _tmp2);
-		(_tmp1 = (g_free (_tmp1), NULL));
-		(basename = (g_free (basename), NULL));
+		_tmp1 = (g_free (_tmp1), NULL);
+		basename = (g_free (basename), NULL);
 	}
 	_tmp3 = NULL;
 	return (_tmp3 = self->priv->csource_filename, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
@@ -287,31 +293,24 @@ char* vala_source_file_get_csource_filename (ValaSourceFile* self) {
  * @return C header filename to include
  */
 char* vala_source_file_get_cinclude_filename (ValaSourceFile* self) {
-	const char* _tmp4;
+	const char* _tmp3;
 	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
 	if (self->priv->cinclude_filename == NULL) {
 		char* basename;
 		char* _tmp0;
-		basename = g_strndup (vala_source_file_get_filename (self), ((guint) (g_utf8_strlen (vala_source_file_get_filename (self), -1) - g_utf8_strlen (".vala", -1))));
+		char* _tmp2;
+		char* _tmp1;
+		basename = g_strndup (self->priv->_filename, ((gulong) ((guint) (g_utf8_strlen (self->priv->_filename, -1) - g_utf8_strlen (".vala", -1)))));
 		_tmp0 = NULL;
 		basename = (_tmp0 = g_path_get_basename (basename), (basename = (g_free (basename), NULL)), _tmp0);
-		if (vala_code_context_get_basedir (vala_source_file_get_context (self)) == NULL && vala_code_context_get_library (vala_source_file_get_context (self)) != NULL) {
-			char* _tmp1;
-			/* backward-compatibility*/
-			_tmp1 = NULL;
-			self->priv->cinclude_filename = (_tmp1 = g_strdup_printf ("%s/%s.h", vala_code_context_get_library (vala_source_file_get_context (self)), basename), (self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL)), _tmp1);
-		} else {
-			char* _tmp3;
-			char* _tmp2;
-			_tmp3 = NULL;
-			_tmp2 = NULL;
-			self->priv->cinclude_filename = (_tmp3 = g_strdup_printf ("%s%s.h", (_tmp2 = vala_source_file_get_subdir (self)), basename), (self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL)), _tmp3);
-			(_tmp2 = (g_free (_tmp2), NULL));
-		}
-		(basename = (g_free (basename), NULL));
+		_tmp2 = NULL;
+		_tmp1 = NULL;
+		self->priv->cinclude_filename = (_tmp2 = g_strdup_printf ("%s%s.h", (_tmp1 = vala_source_file_get_subdir (self)), basename), (self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL)), _tmp2);
+		_tmp1 = (g_free (_tmp1), NULL);
+		basename = (g_free (basename), NULL);
 	}
-	_tmp4 = NULL;
-	return (_tmp4 = self->priv->cinclude_filename, (_tmp4 == NULL ? NULL : g_strdup (_tmp4)));
+	_tmp3 = NULL;
+	return (_tmp3 = self->priv->cinclude_filename, (_tmp3 == NULL ? NULL : g_strdup (_tmp3)));
 }
 
 
@@ -325,26 +324,28 @@ char* vala_source_file_get_cinclude_filename (ValaSourceFile* self) {
 void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* sym, ValaSourceFileDependencyType dep_type) {
 	ValaSymbol* s;
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
-	g_return_if_fail (VALA_IS_SYMBOL (sym));
-	if (vala_source_file_get_pkg (self)) {
+	g_return_if_fail (sym == NULL || VALA_IS_SYMBOL (sym));
+	if (self->priv->_pkg) {
 		return;
 	}
 	s = NULL;
-	if (VALA_IS_DATA_TYPE (sym) || VALA_IS_METHOD (sym) || VALA_IS_FIELD (sym) || VALA_IS_PROPERTY (sym) || VALA_IS_CONSTANT (sym)) {
+	if (VALA_IS_TYPESYMBOL (sym) || VALA_IS_METHOD (sym) || VALA_IS_FIELD (sym) || VALA_IS_PROPERTY (sym) || VALA_IS_CONSTANT (sym)) {
+		ValaSymbol* _tmp1;
 		ValaSymbol* _tmp0;
+		_tmp1 = NULL;
 		_tmp0 = NULL;
-		s = (_tmp0 = g_object_ref (sym), (s == NULL ? NULL : (s = (g_object_unref (s), NULL))), _tmp0);
+		s = (_tmp1 = (_tmp0 = sym, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (s == NULL ? NULL : (s = (g_object_unref (s), NULL))), _tmp1);
 	} else {
 		if (VALA_IS_FORMAL_PARAMETER (sym)) {
-			ValaFormalParameter* _tmp1;
+			ValaFormalParameter* _tmp2;
 			ValaFormalParameter* fp;
-			ValaSymbol* _tmp3;
-			ValaDataType* _tmp2;
-			_tmp1 = NULL;
-			fp = (_tmp1 = VALA_FORMAL_PARAMETER (sym), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1)));
-			_tmp3 = NULL;
+			ValaSymbol* _tmp4;
+			ValaTypesymbol* _tmp3;
 			_tmp2 = NULL;
-			s = (_tmp3 = VALA_SYMBOL ((_tmp2 = vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (fp)), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)))), (s == NULL ? NULL : (s = (g_object_unref (s), NULL))), _tmp3);
+			fp = (_tmp2 = VALA_FORMAL_PARAMETER (sym), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2)));
+			_tmp4 = NULL;
+			_tmp3 = NULL;
+			s = (_tmp4 = VALA_SYMBOL ((_tmp3 = vala_data_type_get_data_type (vala_formal_parameter_get_type_reference (fp)), (_tmp3 == NULL ? NULL : g_object_ref (_tmp3)))), (s == NULL ? NULL : (s = (g_object_unref (s), NULL))), _tmp4);
 			if (s == NULL) {
 				(fp == NULL ? NULL : (fp = (g_object_unref (fp), NULL)));
 				(s == NULL ? NULL : (s = (g_object_unref (s), NULL)));
@@ -358,7 +359,7 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 	}
 	/* generic type parameter */
 	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE) {
-		if (vala_source_file_get_pkg (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (s))))) {
+		if (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (s)))->priv->_pkg) {
 			{
 				GeeCollection* fn_collection;
 				GeeIterator* fn_it;
@@ -366,10 +367,10 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 				fn_it = gee_iterable_iterator (GEE_ITERABLE (fn_collection));
 				while (gee_iterator_next (fn_it)) {
 					char* fn;
-					fn = gee_iterator_get (fn_it);
+					fn = ((char*) gee_iterator_get (fn_it));
 					{
 						gee_collection_add (GEE_COLLECTION (self->priv->source_external_includes), fn);
-						(fn = (g_free (fn), NULL));
+						fn = (g_free (fn), NULL);
 					}
 				}
 				(fn_collection == NULL ? NULL : (fn_collection = (g_object_unref (fn_collection), NULL)));
@@ -383,10 +384,10 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 				fn_it = gee_iterable_iterator (GEE_ITERABLE (fn_collection));
 				while (gee_iterator_next (fn_it)) {
 					char* fn;
-					fn = gee_iterator_get (fn_it);
+					fn = ((char*) gee_iterator_get (fn_it));
 					{
 						gee_collection_add (GEE_COLLECTION (self->priv->source_internal_includes), fn);
-						(fn = (g_free (fn), NULL));
+						fn = (g_free (fn), NULL);
 					}
 				}
 				(fn_collection == NULL ? NULL : (fn_collection = (g_object_unref (fn_collection), NULL)));
@@ -396,7 +397,7 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 		(s == NULL ? NULL : (s = (g_object_unref (s), NULL)));
 		return;
 	}
-	if (vala_source_file_get_pkg (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (s))))) {
+	if (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (s)))->priv->_pkg) {
 		{
 			GeeCollection* fn_collection;
 			GeeIterator* fn_it;
@@ -404,11 +405,11 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 			fn_it = gee_iterable_iterator (GEE_ITERABLE (fn_collection));
 			while (gee_iterator_next (fn_it)) {
 				char* fn;
-				fn = gee_iterator_get (fn_it);
+				fn = ((char*) gee_iterator_get (fn_it));
 				{
 					/* external package */
 					gee_collection_add (GEE_COLLECTION (self->priv->header_external_includes), fn);
-					(fn = (g_free (fn), NULL));
+					fn = (g_free (fn), NULL);
 				}
 			}
 			(fn_collection == NULL ? NULL : (fn_collection = (g_object_unref (fn_collection), NULL)));
@@ -417,7 +418,7 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 		(s == NULL ? NULL : (s = (g_object_unref (s), NULL)));
 		return;
 	}
-	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL || (VALA_IS_DATA_TYPE (s) && !vala_data_type_is_reference_type ((VALA_DATA_TYPE (s))))) {
+	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL || (VALA_IS_TYPESYMBOL (s) && !vala_typesymbol_is_reference_type ((VALA_TYPESYMBOL (s))))) {
 		{
 			GeeCollection* fn_collection;
 			GeeIterator* fn_it;
@@ -425,10 +426,10 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 			fn_it = gee_iterable_iterator (GEE_ITERABLE (fn_collection));
 			while (gee_iterator_next (fn_it)) {
 				char* fn;
-				fn = gee_iterator_get (fn_it);
+				fn = ((char*) gee_iterator_get (fn_it));
 				{
 					gee_collection_add (GEE_COLLECTION (self->priv->header_internal_includes), fn);
-					(fn = (g_free (fn), NULL));
+					fn = (g_free (fn), NULL);
 				}
 			}
 			(fn_collection == NULL ? NULL : (fn_collection = (g_object_unref (fn_collection), NULL)));
@@ -441,6 +442,35 @@ void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* s
 }
 
 
+/**
+ * Adds the symbols that define the specified type to the list of
+ * symbols code in this source file depends on.
+ *
+ * @param type     a data type
+ * @param dep_type type of dependency
+ */
+void vala_source_file_add_type_dependency (ValaSourceFile* self, ValaDataType* type, ValaSourceFileDependencyType dep_type) {
+	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
+	g_return_if_fail (type == NULL || VALA_IS_DATA_TYPE (type));
+	{
+		GeeCollection* type_symbol_collection;
+		GeeIterator* type_symbol_it;
+		type_symbol_collection = vala_data_type_get_symbols (type);
+		type_symbol_it = gee_iterable_iterator (GEE_ITERABLE (type_symbol_collection));
+		while (gee_iterator_next (type_symbol_it)) {
+			ValaSymbol* type_symbol;
+			type_symbol = ((ValaSymbol*) gee_iterator_get (type_symbol_it));
+			{
+				vala_source_file_add_symbol_dependency (self, type_symbol, dep_type);
+				(type_symbol == NULL ? NULL : (type_symbol = (g_object_unref (type_symbol), NULL)));
+			}
+		}
+		(type_symbol_collection == NULL ? NULL : (type_symbol_collection = (g_object_unref (type_symbol_collection), NULL)));
+		(type_symbol_it == NULL ? NULL : (type_symbol_it = (g_object_unref (type_symbol_it), NULL)));
+	}
+}
+
+
 /**
  * Returns the list of external includes the generated C header file
  * requires.
@@ -463,10 +493,9 @@ void vala_source_file_add_header_internal_include (ValaSourceFile* self, const c
 	char* _tmp0;
 	gboolean _tmp1;
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
-	g_return_if_fail (include != NULL);
 	/* skip includes to self */
 	_tmp0 = NULL;
-	if ((_tmp1 = g_utf8_collate (include, (_tmp0 = vala_source_file_get_cinclude_filename (self))) != 0, (_tmp0 = (g_free (_tmp0), NULL)), _tmp1)) {
+	if ((_tmp1 = _vala_strcmp0 (include, (_tmp0 = vala_source_file_get_cinclude_filename (self))) != 0, (_tmp0 = (g_free (_tmp0), NULL)), _tmp1)) {
 		gee_collection_add (GEE_COLLECTION (self->priv->header_internal_includes), include);
 	}
 }
@@ -532,6 +561,80 @@ GeeCollection* vala_source_file_get_header_internal_dependencies (ValaSourceFile
 }
 
 
+/**
+ * Returns the requested line from this file, loading it if needed.
+ *
+ * @param lineno 1-based line number
+ * @return       the specified source line
+ */
+char* vala_source_file_get_source_line (ValaSourceFile* self, gint lineno) {
+	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
+	if (self->priv->source_array == NULL) {
+		vala_source_file_read_source_file (self);
+	}
+	if (lineno < 1 || lineno > gee_collection_get_size (GEE_COLLECTION (self->priv->source_array))) {
+		return NULL;
+	}
+	return ((char*) gee_list_get (GEE_LIST (self->priv->source_array), lineno - 1));
+}
+
+
+/**
+ * Parses the input file into ::source_array.
+ */
+static void vala_source_file_read_source_file (ValaSourceFile* self) {
+	GError * inner_error;
+	char* cont;
+	GeeArrayList* _tmp0;
+	char** _tmp3;
+	gint lines_length1;
+	char** lines;
+	guint idx;
+	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
+	inner_error = NULL;
+	cont = NULL;
+	_tmp0 = NULL;
+	self->priv->source_array = (_tmp0 = gee_array_list_new (G_TYPE_STRING, ((GBoxedCopyFunc) g_strdup), g_free, g_direct_equal), (self->priv->source_array == NULL ? NULL : (self->priv->source_array = (g_object_unref (self->priv->source_array), NULL))), _tmp0);
+	{
+		gboolean _tmp2;
+		char* _tmp1;
+		_tmp1 = NULL;
+		_tmp2 = g_file_get_contents (self->priv->_filename, &_tmp1, NULL, &inner_error);
+		(cont = (g_free (cont), NULL));
+		cont = _tmp1;
+		_tmp2;
+		if (inner_error != NULL) {
+			if (inner_error->domain == G_FILE_ERROR) {
+				goto __catch0_g_file_error;
+			}
+			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+			g_clear_error (&inner_error);
+		}
+	}
+	goto __finally0;
+	__catch0_g_file_error:
+	{
+		GError * fe;
+		fe = inner_error;
+		inner_error = NULL;
+		{
+			cont = (g_free (cont), NULL);
+			return;
+		}
+	}
+	__finally0:
+	;
+	_tmp3 = NULL;
+	lines = (_tmp3 = g_strsplit (cont, "\n", 0), lines_length1 = -1, _tmp3);
+	idx = 0U;
+	for (idx = ((guint) 0); lines[idx] != NULL; (idx = idx + 1)) {
+		gee_collection_add (GEE_COLLECTION (self->priv->source_array), lines[idx]);
+	}
+	cont = (g_free (cont), NULL);
+	lines = (_vala_array_free (lines, lines_length1, ((GDestroyNotify) g_free)), NULL);
+}
+
+
 char* vala_source_file_get_filename (ValaSourceFile* self) {
 	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
 	return self->priv->_filename;
@@ -539,10 +642,12 @@ char* vala_source_file_get_filename (ValaSourceFile* self) {
 
 
 void vala_source_file_set_filename (ValaSourceFile* self, const char* value) {
-	char* _tmp1;
+	char* _tmp2;
+	const char* _tmp1;
 	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
+	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_filename = (_tmp1 = g_strdup (value), (self->priv->_filename = (g_free (self->priv->_filename), NULL)), _tmp1);
+	self->priv->_filename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_filename = (g_free (self->priv->_filename), NULL)), _tmp2);
 }
 
 
@@ -719,26 +824,28 @@ static void vala_source_file_init (ValaSourceFile * self) {
 	self->priv->source_internal_includes = GEE_LIST (gee_array_list_new (G_TYPE_STRING, ((GBoxedCopyFunc) g_strdup), g_free, g_direct_equal));
 	self->priv->header_internal_full_dependencies = GEE_LIST (gee_array_list_new (VALA_TYPE_SOURCE_FILE, NULL, NULL, g_direct_equal));
 	self->priv->header_internal_dependencies = GEE_LIST (gee_array_list_new (VALA_TYPE_SOURCE_FILE, NULL, NULL, g_direct_equal));
+	self->priv->source_array = NULL;
 }
 
 
 static void vala_source_file_dispose (GObject * obj) {
 	ValaSourceFile * self;
 	self = VALA_SOURCE_FILE (obj);
-	(self->priv->_filename = (g_free (self->priv->_filename), NULL));
-	(self->priv->_comment = (g_free (self->priv->_comment), NULL));
+	self->priv->_filename = (g_free (self->priv->_filename), NULL);
+	self->priv->_comment = (g_free (self->priv->_comment), NULL);
 	(self->priv->_cycle == NULL ? NULL : (self->priv->_cycle = (g_object_unref (self->priv->_cycle), NULL)));
 	(self->priv->using_directives == NULL ? NULL : (self->priv->using_directives = (g_object_unref (self->priv->using_directives), NULL)));
 	(self->priv->nodes == NULL ? NULL : (self->priv->nodes = (g_object_unref (self->priv->nodes), NULL)));
-	(self->priv->cheader_filename = (g_free (self->priv->cheader_filename), NULL));
-	(self->priv->csource_filename = (g_free (self->priv->csource_filename), NULL));
-	(self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL));
+	self->priv->cheader_filename = (g_free (self->priv->cheader_filename), NULL);
+	self->priv->csource_filename = (g_free (self->priv->csource_filename), NULL);
+	self->priv->cinclude_filename = (g_free (self->priv->cinclude_filename), NULL);
 	(self->priv->header_external_includes == NULL ? NULL : (self->priv->header_external_includes = (g_object_unref (self->priv->header_external_includes), NULL)));
 	(self->priv->header_internal_includes == NULL ? NULL : (self->priv->header_internal_includes = (g_object_unref (self->priv->header_internal_includes), NULL)));
 	(self->priv->source_external_includes == NULL ? NULL : (self->priv->source_external_includes = (g_object_unref (self->priv->source_external_includes), NULL)));
 	(self->priv->source_internal_includes == NULL ? NULL : (self->priv->source_internal_includes = (g_object_unref (self->priv->source_internal_includes), NULL)));
 	(self->priv->header_internal_full_dependencies == NULL ? NULL : (self->priv->header_internal_full_dependencies = (g_object_unref (self->priv->header_internal_full_dependencies), NULL)));
 	(self->priv->header_internal_dependencies == NULL ? NULL : (self->priv->header_internal_dependencies = (g_object_unref (self->priv->header_internal_dependencies), NULL)));
+	(self->priv->source_array == NULL ? NULL : (self->priv->source_array = (g_object_unref (self->priv->source_array), NULL)));
 	G_OBJECT_CLASS (vala_source_file_parent_class)->dispose (obj);
 }
 
@@ -754,4 +861,43 @@ GType vala_source_file_get_type (void) {
 
 
 
+GType vala_source_file_dependency_type_get_type (void) {
+	static GType vala_source_file_dependency_type_type_id = 0;
+	if (G_UNLIKELY (vala_source_file_dependency_type_type_id == 0)) {
+		static const GEnumValue values[] = {{VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL, "VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL", "header-full"}, {VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW, "VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW", "header-shallow"}, {VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE, "VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE", "source"}, {0, NULL, NULL}};
+		vala_source_file_dependency_type_type_id = g_enum_register_static ("ValaSourceFileDependencyType", values);
+	}
+	return vala_source_file_dependency_type_type_id;
+}
+
+
+static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+	if (array != NULL && destroy_func != NULL) {
+		int i;
+		if (array_length >= 0)
+		for (i = 0; i < array_length; i = i + 1) {
+			if (((gpointer*) array)[i] != NULL)
+			destroy_func (((gpointer*) array)[i]);
+		}
+		else
+		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
+			destroy_func (((gpointer*) array)[i]);
+		}
+	}
+	g_free (array);
+}
+
+
+static int _vala_strcmp0 (const char * str1, const char * str2) {
+	if (str1 == NULL) {
+		return -(str1 != str2);
+	}
+	if (str2 == NULL) {
+		return (str1 != str2);
+	}
+	return strcmp (str1, str2);
+}
+
+
+
 
