@@ -45,11 +45,11 @@ typedef struct _ValaSwitchSectionPrivate ValaSwitchSectionPrivate;
  * Represents a switch section in the source code.
  */
 struct _ValaSwitchSection {
-	ValaBlock parent;
+	ValaBlock parent_instance;
 	ValaSwitchSectionPrivate * priv;
 };
 struct _ValaSwitchSectionClass {
-	ValaBlockClass parent;
+	ValaBlockClass parent_class;
 };
 
 ValaSwitchSection* vala_switch_section_new (ValaSourceReference* source_reference);
@@ -60,6 +60,7 @@ void vala_switch_section_add_statement (ValaSwitchSection* self, ValaStatement*
 GeeCollection* vala_switch_section_get_statements (ValaSwitchSection* self);
 GType vala_switch_section_get_type (void);
 
+
 G_END_DECLS
 
 #endif
