@@ -23,6 +23,10 @@
 #include <vala/valasymbol.h>
 #include <gee/readonlycollection.h>
 #include <vala/valascope.h>
+#include <vala/valanamespace.h>
+
+
+
 
 struct _ValaSymbolPrivate {
 	char* _name;
@@ -52,6 +56,7 @@ static gpointer vala_symbol_parent_class = NULL;
 static void vala_symbol_dispose (GObject * obj);
 
 
+
 /**
  * Check if this symbol is just internal API (and therefore doesn't need 
  * to be listed in header files for instance) by traversing parent symbols
@@ -71,7 +76,7 @@ gboolean vala_symbol_is_internal_symbol (ValaSymbol* self) {
 		_tmp1 = NULL;
 		_tmp0 = NULL;
 		for (sym = (_tmp1 = (_tmp0 = self, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), _tmp1); NULL != sym; sym = (_tmp3 = (_tmp2 = vala_symbol_get_parent_symbol (sym), (_tmp2 == NULL ? NULL : g_object_ref (_tmp2))), (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), _tmp3)) {
-			if (VALA_SYMBOL_ACCESSIBILITY_PRIVATE == vala_symbol_get_access (sym)) {
+			if (VALA_SYMBOL_ACCESSIBILITY_PRIVATE == sym->priv->_access) {
 				gboolean _tmp4;
 				return (_tmp4 = TRUE, (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), _tmp4);
 			}
@@ -97,20 +102,20 @@ char* vala_symbol_get_full_name (ValaSymbol* self) {
 	if (vala_symbol_get_parent_symbol (self) == NULL) {
 		const char* _tmp0;
 		_tmp0 = NULL;
-		return (_tmp0 = vala_symbol_get_name (self), (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
+		return (_tmp0 = self->priv->_name, (_tmp0 == NULL ? NULL : g_strdup (_tmp0)));
 	}
-	if (vala_symbol_get_name (self) == NULL) {
+	if (self->priv->_name == NULL) {
 		return vala_symbol_get_full_name (vala_symbol_get_parent_symbol (self));
 	}
 	_tmp3 = NULL;
 	if ((_tmp4 = (_tmp3 = vala_symbol_get_full_name (vala_symbol_get_parent_symbol (self))) == NULL, (_tmp3 = (g_free (_tmp3), NULL)), _tmp4)) {
 		const char* _tmp5;
 		_tmp5 = NULL;
-		return (_tmp5 = vala_symbol_get_name (self), (_tmp5 == NULL ? NULL : g_strdup (_tmp5)));
+		return (_tmp5 = self->priv->_name, (_tmp5 == NULL ? NULL : g_strdup (_tmp5)));
 	}
 	_tmp7 = NULL;
 	_tmp8 = NULL;
-	return (_tmp8 = g_strdup_printf ("%s.%s", (_tmp7 = vala_symbol_get_full_name (vala_symbol_get_parent_symbol (self))), vala_symbol_get_name (self)), (_tmp7 = (g_free (_tmp7), NULL)), _tmp8);
+	return (_tmp8 = g_strdup_printf ("%s.%s", (_tmp7 = vala_symbol_get_full_name (vala_symbol_get_parent_symbol (self))), self->priv->_name), (_tmp7 = (g_free (_tmp7), NULL)), _tmp8);
 }
 
 
@@ -122,12 +127,12 @@ char* vala_symbol_get_full_name (ValaSymbol* self) {
  */
 static char* vala_symbol_real_get_cprefix (ValaSymbol* self) {
 	g_return_val_if_fail (VALA_IS_SYMBOL (self), NULL);
-	if (vala_symbol_get_name (self) == NULL) {
+	if (self->priv->_name == NULL) {
 		return g_strdup ("");
 	} else {
 		const char* _tmp1;
 		_tmp1 = NULL;
-		return (_tmp1 = vala_symbol_get_name (self), (_tmp1 == NULL ? NULL : g_strdup (_tmp1)));
+		return (_tmp1 = self->priv->_name, (_tmp1 == NULL ? NULL : g_strdup (_tmp1)));
 	}
 }
 
@@ -240,6 +245,32 @@ char* vala_symbol_camel_case_to_lower_case (const char* camel_case) {
 }
 
 
+/* get the top scope from where this symbol is still accessible*/
+ValaScope* vala_symbol_get_top_accessible_scope (ValaSymbol* self) {
+	g_return_val_if_fail (VALA_IS_SYMBOL (self), NULL);
+	if (self->priv->_access != VALA_SYMBOL_ACCESSIBILITY_PUBLIC) {
+		ValaScope* _tmp0;
+		ValaScope* scope;
+		/* private symbols are accessible within the scope where the symbol has been declared*/
+		_tmp0 = NULL;
+		scope = (_tmp0 = vala_symbol_get_owner (self), (_tmp0 == NULL ? NULL : g_object_ref (_tmp0)));
+		while (scope != NULL && VALA_IS_NAMESPACE (vala_scope_get_owner (scope)) && vala_scope_get_owner (scope)->priv->_name != NULL) {
+			ValaScope* _tmp2;
+			ValaScope* _tmp1;
+			/* skip scopes of normal namespaces, they don't influence accessibility*/
+			_tmp2 = NULL;
+			_tmp1 = NULL;
+			scope = (_tmp2 = (_tmp1 = vala_scope_get_parent_scope (scope), (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (scope == NULL ? NULL : (scope = (g_object_unref (scope), NULL))), _tmp2);
+		}
+		return scope;
+	}
+	if (vala_symbol_get_parent_symbol (self) == NULL) {
+		return NULL;
+	}
+	return vala_symbol_get_top_accessible_scope (vala_symbol_get_parent_symbol (self));
+}
+
+
 ValaSymbol* vala_symbol_get_parent_symbol (ValaSymbol* self) {
 	g_return_val_if_fail (VALA_IS_SYMBOL (self), NULL);
 	if (vala_symbol_get_owner (self) == NULL) {
@@ -357,7 +388,7 @@ static void vala_symbol_get_property (GObject * object, guint property_id, GValu
 		g_value_set_boolean (value, vala_symbol_get_active (self));
 		break;
 		case VALA_SYMBOL_ACCESS:
-		g_value_set_int (value, vala_symbol_get_access (self));
+		g_value_set_enum (value, vala_symbol_get_access (self));
 		break;
 		case VALA_SYMBOL_SCOPE:
 		g_value_set_object (value, vala_symbol_get_scope (self));
@@ -386,7 +417,7 @@ static void vala_symbol_set_property (GObject * object, guint property_id, const
 		vala_symbol_set_active (self, g_value_get_boolean (value));
 		break;
 		case VALA_SYMBOL_ACCESS:
-		vala_symbol_set_access (self, g_value_get_int (value));
+		vala_symbol_set_access (self, g_value_get_enum (value));
 		break;
 		case VALA_SYMBOL_IS_IMPORTED:
 		vala_symbol_set_is_imported (self, g_value_get_boolean (value));
@@ -413,7 +444,7 @@ static void vala_symbol_class_init (ValaSymbolClass * klass) {
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_OWNER, g_param_spec_object ("owner", "owner", "owner", VALA_TYPE_SCOPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_NAME, g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_ACTIVE, g_param_spec_boolean ("active", "active", "active", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_ACCESS, g_param_spec_int ("access", "access", "access", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_ACCESS, g_param_spec_enum ("access", "access", "access", VALA_TYPE_SYMBOL_ACCESSIBILITY, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_SCOPE, g_param_spec_object ("scope", "scope", "scope", VALA_TYPE_SCOPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SYMBOL_IS_IMPORTED, g_param_spec_boolean ("is-imported", "is-imported", "is-imported", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
@@ -444,4 +475,15 @@ GType vala_symbol_get_type (void) {
 
 
 
+GType vala_symbol_accessibility_get_type (void) {
+	static GType vala_symbol_accessibility_type_id = 0;
+	if (G_UNLIKELY (vala_symbol_accessibility_type_id == 0)) {
+		static const GEnumValue values[] = {{VALA_SYMBOL_ACCESSIBILITY_PRIVATE, "VALA_SYMBOL_ACCESSIBILITY_PRIVATE", "private"}, {VALA_SYMBOL_ACCESSIBILITY_INTERNAL, "VALA_SYMBOL_ACCESSIBILITY_INTERNAL", "internal"}, {VALA_SYMBOL_ACCESSIBILITY_PROTECTED, "VALA_SYMBOL_ACCESSIBILITY_PROTECTED", "protected"}, {VALA_SYMBOL_ACCESSIBILITY_PUBLIC, "VALA_SYMBOL_ACCESSIBILITY_PUBLIC", "public"}, {0, NULL, NULL}};
+		vala_symbol_accessibility_type_id = g_enum_register_static ("ValaSymbolAccessibility", values);
+	}
+	return vala_symbol_accessibility_type_id;
+}
+
+
+
 
