@@ -25,6 +25,9 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaThrowStatementPrivate {
 	ValaExpression* _error_expression;
 };
@@ -35,12 +38,13 @@ enum  {
 };
 static void vala_throw_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
 static void vala_throw_statement_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
-static void vala_throw_statement_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node);
+static void vala_throw_statement_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node);
 static gpointer vala_throw_statement_parent_class = NULL;
 static ValaStatementIface* vala_throw_statement_vala_statement_parent_iface = NULL;
 static void vala_throw_statement_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new throw statement.
  *
@@ -50,7 +54,7 @@ static void vala_throw_statement_dispose (GObject * obj);
  */
 ValaThrowStatement* vala_throw_statement_new (ValaExpression* error_expression, ValaSourceReference* source_reference) {
 	ValaThrowStatement * self;
-	g_return_val_if_fail (VALA_IS_EXPRESSION (error_expression), NULL);
+	g_return_val_if_fail (error_expression == NULL || VALA_IS_EXPRESSION (error_expression), NULL);
 	g_return_val_if_fail (source_reference == NULL || VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
 	self = g_object_newv (VALA_TYPE_THROW_STATEMENT, 0, NULL);
 	vala_throw_statement_set_error_expression (self, error_expression);
@@ -62,7 +66,7 @@ ValaThrowStatement* vala_throw_statement_new (ValaExpression* error_expression,
 static void vala_throw_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaThrowStatement * self;
 	self = VALA_THROW_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	vala_code_visitor_visit_throw_statement (visitor, self);
 }
 
@@ -70,20 +74,20 @@ static void vala_throw_statement_real_accept (ValaCodeNode* base, ValaCodeVisito
 static void vala_throw_statement_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
 	ValaThrowStatement * self;
 	self = VALA_THROW_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
+	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
 	if (vala_throw_statement_get_error_expression (self) != NULL) {
 		vala_code_node_accept (VALA_CODE_NODE (vala_throw_statement_get_error_expression (self)), visitor);
 	}
 }
 
 
-static void vala_throw_statement_real_replace (ValaCodeNode* base, ValaCodeNode* old_node, ValaCodeNode* new_node) {
+static void vala_throw_statement_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node) {
 	ValaThrowStatement * self;
 	self = VALA_THROW_STATEMENT (base);
-	g_return_if_fail (VALA_IS_CODE_NODE (old_node));
-	g_return_if_fail (VALA_IS_CODE_NODE (new_node));
-	if (VALA_CODE_NODE (vala_throw_statement_get_error_expression (self)) == old_node) {
-		vala_throw_statement_set_error_expression (self, VALA_EXPRESSION (new_node));
+	g_return_if_fail (old_node == NULL || VALA_IS_EXPRESSION (old_node));
+	g_return_if_fail (new_node == NULL || VALA_IS_EXPRESSION (new_node));
+	if (vala_throw_statement_get_error_expression (self) == old_node) {
+		vala_throw_statement_set_error_expression (self, new_node);
 	}
 }
 
@@ -143,7 +147,7 @@ static void vala_throw_statement_class_init (ValaThrowStatementClass * klass) {
 	G_OBJECT_CLASS (klass)->dispose = vala_throw_statement_dispose;
 	VALA_CODE_NODE_CLASS (klass)->accept = vala_throw_statement_real_accept;
 	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_throw_statement_real_accept_children;
-	VALA_CODE_NODE_CLASS (klass)->replace = vala_throw_statement_real_replace;
+	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_throw_statement_real_replace_expression;
 	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_THROW_STATEMENT_ERROR_EXPRESSION, g_param_spec_object ("error-expression", "error-expression", "error-expression", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
