@@ -29,6 +29,9 @@
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
 
+
+
+
 struct _ValaTryStatementPrivate {
 	ValaBlock* _body;
 	ValaBlock* _finally_body;
@@ -47,6 +50,7 @@ static ValaStatementIface* vala_try_statement_vala_statement_parent_iface = NULL
 static void vala_try_statement_dispose (GObject * obj);
 
 
+
 /**
  * Creates a new try statement.
  *
@@ -103,7 +107,7 @@ static void vala_try_statement_real_accept_children (ValaCodeNode* base, ValaCod
 	ValaTryStatement * self;
 	self = VALA_TRY_STATEMENT (base);
 	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
-	vala_code_node_accept (VALA_CODE_NODE (vala_try_statement_get_body (self)), visitor);
+	vala_code_node_accept (VALA_CODE_NODE (self->priv->_body), visitor);
 	{
 		GeeList* clause_collection;
 		GeeIterator* clause_it;
@@ -119,8 +123,8 @@ static void vala_try_statement_real_accept_children (ValaCodeNode* base, ValaCod
 		}
 		(clause_it == NULL ? NULL : (clause_it = (g_object_unref (clause_it), NULL)));
 	}
-	if (vala_try_statement_get_finally_body (self) != NULL) {
-		vala_code_node_accept (VALA_CODE_NODE (vala_try_statement_get_finally_body (self)), visitor);
+	if (self->priv->_finally_body != NULL) {
+		vala_code_node_accept (VALA_CODE_NODE (self->priv->_finally_body), visitor);
 	}
 }
 
